@@ -28,18 +28,6 @@
 int afs_osicred_initialized = 0;
 afs_ucred_t afs_osi_cred;
 
-void
-afs_osi_SetTime(osi_timeval_t * tvp)
-{
-    struct timespec tv;
-    tv.tv_sec = tvp->tv_sec;
-    tv.tv_nsec = tvp->tv_usec * NSEC_PER_USEC;
-
-    AFS_STATCNT(osi_SetTime);
-
-    do_settimeofday(&tv);
-}
-
 void
 osi_linux_mask(void)
 {
@@ -49,6 +37,17 @@ osi_linux_mask(void)
     SIG_UNLOCK(current);
 }
 
+void
+osi_linux_unmaskrxk(void)
+{
+    extern struct task_struct *rxk_ListenerTask;
+    /* Note this unblocks signals on the rxk_Listener
+     * thread from a thread that is stopping rxk */
+    SIG_LOCK(rxk_ListenerTask);
+    sigdelset(&rxk_ListenerTask->blocked, SIGKILL);
+    SIG_UNLOCK(rxk_ListenerTask);
+}
+
 /* LOOKUP_POSITIVE is becoming the default */
 #ifndef LOOKUP_POSITIVE
 #define LOOKUP_POSITIVE 0
@@ -65,7 +64,6 @@ osi_lookupname_internal(char *aname, int followlink, struct vfsmount **mnt,
     afs_linux_path_t path_data;
 #endif
     int flags = LOOKUP_POSITIVE;
-    code = ENOENT;
 
     if (followlink)
        flags |= LOOKUP_FOLLOW;
@@ -113,7 +111,6 @@ osi_lookupname(char *aname, uio_seg_t seg, int followlink,
     int code;
     char *name;
 
-    code = ENOENT;
     if (seg == AFS_UIOUSER) {
 	name = afs_getname(aname);
 	if (IS_ERR(name))
@@ -136,14 +133,13 @@ int osi_abspath(char *aname, char *buf, int buflen,
     char *name, *path;
     int code;
 
-    code = ENOENT;
     name = afs_getname(aname);
     if (IS_ERR(name))
 	return -PTR_ERR(name);
     code = osi_lookupname_internal(name, followlink, &mnt, &dp);
     if (!code) {
 #if defined(D_PATH_TAKES_STRUCT_PATH)
-	afs_linux_path_t p = { mnt, dp };
+	afs_linux_path_t p = { .mnt = mnt, .dentry = dp };
 	path = d_path(&p, buf, buflen);
 #else
 	path = d_path(dp, mnt, buf, buflen);
