@@ -124,12 +124,21 @@ afs_csdb_open(struct inode *inode, struct file *file)
     return seq_open(file, &afs_csdb_op);
 }
 
+#if defined(HAVE_LINUX_STRUCT_PROC_OPS)
+static struct proc_ops afs_csdb_operations = {
+    .proc_open    = afs_csdb_open,
+    .proc_read    = seq_read,
+    .proc_lseek  = seq_lseek,
+    .proc_release = seq_release,
+};
+#else
 static struct file_operations afs_csdb_operations = {
     .open    = afs_csdb_open,
     .read    = seq_read,
     .llseek  = seq_lseek,
     .release = seq_release,
 };
+#endif /* HAVE_LINUX_STRUCT_PROC_OPS */
 
 static void *
 uu_start(struct seq_file *m, loff_t *pos)
@@ -195,6 +204,7 @@ uu_show(struct seq_file *m, void *p)
 {
     struct cell *tc = 0;
     struct unixuser *tu = p;
+    union tokenUnion *token;
     char *cellname;
 
     if (p == (void *)1) {
@@ -210,6 +220,11 @@ uu_show(struct seq_file *m, void *p)
 
     AFS_GLOCK();
 
+    tu->refCount++;
+    ReleaseReadLock(&afs_xuser);
+
+    afs_LockUser(tu, READ_LOCK, 0);
+
     if (tu->cell == -1) {
 	cellname = "<default>";
     } else {
@@ -219,14 +234,17 @@ uu_show(struct seq_file *m, void *p)
     }
 
     seq_printf(m, "%10d %4d %04x    %-25s %10d",
-	       tu->uid, tu->refCount, tu->states, cellname, tu->vid);
+	       tu->uid, tu->refCount, tu->states, cellname, tu->viceId);
 
     if (tc) afs_PutCell(tc, READ_LOCK);
 
     if (tu->states & UHasTokens) {
+	token = afs_FindToken(tu->tokens, RX_SECIDX_KAD);
 	seq_printf(m, "  %10d %10d %10d %3d",
-		   tu->tokenTime, tu->ct.BeginTimestamp, tu->ct.EndTimestamp,
-		   tu->ct.AuthHandle);
+		   tu->tokenTime,
+		   (token!=NULL)?token->rxkad.clearToken.BeginTimestamp:0,
+		   (token!=NULL)?token->rxkad.clearToken.EndTimestamp:0,
+		   (token!=NULL)?token->rxkad.clearToken.AuthHandle:0);
     } else {
 	seq_printf(m, "  %-36s", "Tokens Not Set");
     }
@@ -254,6 +272,9 @@ uu_show(struct seq_file *m, void *p)
     }
     seq_printf(m, "\n");
 
+    afs_PutUser(tu, READ_LOCK);
+    ObtainReadLock(&afs_xuser);
+
     AFS_GUNLOCK();
 
     return 0;
@@ -272,13 +293,21 @@ afs_unixuser_open(struct inode *inode, struct file *file)
     return seq_open(file, &afs_unixuser_seqop);
 }
 
-static struct file_operations afs_unixuser_fops = {
+#if defined(HAVE_LINUX_STRUCT_PROC_OPS)
+static struct proc_ops afs_unixuser_ops = {
+    .proc_open = afs_unixuser_open,
+    .proc_read = seq_read,
+    .proc_lseek = seq_lseek,
+    .proc_release = seq_release,
+};
+#else
+static struct file_operations afs_unixuser_ops = {
     .open    = afs_unixuser_open,
     .read    = seq_read,
     .llseek  = seq_lseek,
     .release = seq_release,
 };
-
+#endif /* HAVE_LINUX_STRUCT_PROC_OPS */
 
 #else /* HAVE_LINUX_SEQ_FILE_H */
 
@@ -369,7 +398,7 @@ osi_proc_init(void)
     openafs_procfs = proc_mkdir(path, NULL);
 #endif
 #ifdef HAVE_LINUX_SEQ_FILE_H
-    entry = afs_proc_create("unixusers", 0, openafs_procfs, &afs_unixuser_fops);
+    entry = afs_proc_create("unixusers", 0, openafs_procfs, &afs_unixuser_ops);
 # if defined(STRUCT_PROC_DIR_ENTRY_HAS_OWNER)
     if (entry)
 	entry->owner = THIS_MODULE;
