@@ -104,7 +104,11 @@ afs_fill_super(struct super_block *sb, void *data, int silent)
    __module_get(THIS_MODULE);
 
     afs_globalVFS = sb;
+#if defined(SB_NOATIME)
+    sb->s_flags |= SB_NOATIME;
+#else
     sb->s_flags |= MS_NOATIME;
+#endif
     sb->s_blocksize = 1024;
     sb->s_blocksize_bits = 10;
     sb->s_magic = AFS_VFSMAGIC;
@@ -121,8 +125,7 @@ afs_fill_super(struct super_block *sb, void *data, int silent)
     sb->s_bdi->ra_pages = 32;
 #else
     /* used for inodes backing_dev_info field, also */
-    afs_backing_dev_info = kmalloc(sizeof(struct backing_dev_info), GFP_NOFS);
-    memset(afs_backing_dev_info, 0, sizeof(struct backing_dev_info));
+    afs_backing_dev_info = kzalloc(sizeof(struct backing_dev_info), GFP_NOFS);
 #if defined(HAVE_LINUX_BDI_INIT)
     code = bdi_init(afs_backing_dev_info);
     if (code)
@@ -219,7 +222,7 @@ afs_root(struct super_block *afsp)
 		    afs_DestroyAttr(vattr);
 		}
 	    } else
-		code = ENOENT;
+		code = EIO;
 	}
 	crfree(credp);
 	afs_DestroyReq(treq);
