@@ -36,10 +36,6 @@ struct brequest afs_brs[NBRS];	/* request structures */
 struct afs_osi_WaitHandle AFS_WaitHandler, AFS_CSWaitHandler;
 static int afs_brs_count = 0;	/* request counter, to service reqs in order */
 
-static int rxepoch_checked = 0;
-#define afs_CheckRXEpoch() {if (rxepoch_checked == 0 && rxkad_EpochWasSet) { \
-	rxepoch_checked = 1; afs_GCUserData(/* force flag */ 1);  } }
-
 /* PAG garbage collection */
 /* We induce a compile error if param.h does not define AFS_GCPAGS */
 afs_int32 afs_gcpags = AFS_GCPAGS;
@@ -140,11 +136,8 @@ afs_Daemon(void)
     afs_int32 last3MinCheck, last10MinCheck, last60MinCheck, lastNMinCheck;
     afs_int32 last1MinCheck, last5MinCheck;
     afs_uint32 lastCBSlotBump;
-    char cs_warned = 0;
 
     AFS_STATCNT(afs_Daemon);
-    last1MinCheck = last3MinCheck = last60MinCheck = last10MinCheck =
-    last5MinCheck = lastNMinCheck = 0;
 
     afs_rootFid.Fid.Volume = 0;
     while (afs_initState < 101)
@@ -181,6 +174,7 @@ afs_Daemon(void)
 	if (afs_nfsexporter)
 	    afs_FlushActiveVcaches(0);	/* flush NFS writes */
 	afs_FlushVCBs(1);	/* flush queued callbacks */
+
 	afs_MaybeWakeupTruncateDaemon();	/* free cache space if have too */
 	rx_CheckPackets();	/* Does RX need more packets? */
 
@@ -195,7 +189,7 @@ afs_Daemon(void)
 	if (last1MinCheck + 60 < now) {
 	    /* things to do every minute */
 	    DFlush();		/* write out dir buffers */
-	    afs_WriteThroughDSlots();	/* write through cacheinfo entries */
+	    (void)afs_WriteThroughDSlots();	/* write through cacheinfo entries */
 	    ObtainWriteLock(&afs_xvcache, 736);
 	    afs_FlushReclaimedVcaches();
 	    ReleaseWriteLock(&afs_xvcache);
@@ -203,7 +197,6 @@ afs_Daemon(void)
 #if 0
 	    afs_StoreDirtyVcaches();
 #endif
-	    afs_CheckRXEpoch();
 	    last1MinCheck = now;
 	}
 
@@ -226,11 +219,6 @@ afs_Daemon(void)
         }
 
 	if (!afs_CheckServerDaemonStarted) {
-	    /* Do the check here if the correct afsd is not installed. */
-	    if (!cs_warned) {
-		cs_warned = 1;
-		afs_warn("Please install afsd with check server daemon.\n");
-	    }
 	    if (lastNMinCheck + afs_probe_interval < now) {
 		/* only check down servers */
 		afs_CheckServers(1, NULL);
@@ -253,7 +241,7 @@ afs_Daemon(void)
 #endif /* else AFS_USERSPACE_IP_ADDR */
 	    if (!afs_CheckServerDaemonStarted)
 		afs_CheckServers(0, NULL);
-	    afs_GCUserData(0);	/* gc old conns */
+	    afs_GCUserData();	/* gc old conns */
 	    /* This is probably the wrong way of doing GC for the various exporters but it will suffice for a while */
 	    for (exporter = root_exported; exporter;
 		 exporter = exporter->exp_next) {
@@ -318,7 +306,7 @@ afs_Daemon(void)
 int
 afs_CheckRootVolume(void)
 {
-    char rootVolName[32];
+    char rootVolName[MAXROOTVOLNAMELEN];
     struct volume *tvp = NULL;
     int usingDynroot = afs_GetDynrootEnable();
     int localcell;
@@ -363,71 +351,14 @@ afs_CheckRootVolume(void)
 		 * count to zero and fs checkv is executed when the current
 		 * directory is /afs.
 		 */
-#ifdef AFS_LINUX20_ENV
-		{
-		    struct vrequest *treq = NULL;
-		    struct vattr vattr;
-		    cred_t *credp;
-		    struct dentry *dp;
-		    struct vcache *vcp;
-
-		    afs_rootFid.Fid.Volume = volid;
-		    afs_rootFid.Fid.Vnode = 1;
-		    afs_rootFid.Fid.Unique = 1;
-
-		    credp = crref();
-		    if (afs_CreateReq(&treq, credp))
-			goto out;
-		    vcp = afs_GetVCache(&afs_rootFid, treq, NULL, NULL);
-		    if (!vcp)
-			goto out;
-		    afs_getattr(vcp, &vattr, credp);
-		    afs_fill_inode(AFSTOV(vcp), &vattr);
-
-		    dp = d_find_alias(AFSTOV(afs_globalVp));
-
-#if defined(AFS_LINUX24_ENV)
-#if defined(HAVE_DCACHE_LOCK)
-		    spin_lock(&dcache_lock);
+#ifdef AFS_LINUX22_ENV
+		osi_ResetRootVCache(volid);
 #else
-		    spin_lock(&AFSTOV(vcp)->i_lock);
-#endif
-#if defined(AFS_LINUX26_ENV)
-		    spin_lock(&dp->d_lock);
-#endif
-#endif
-#if defined(D_ALIAS_IS_HLIST)
-		    hlist_del_init(&dp->d_alias);
-		    hlist_add_head(&dp->d_alias, &(AFSTOV(vcp)->i_dentry));
-#else
-		    list_del_init(&dp->d_alias);
-		    list_add(&dp->d_alias, &(AFSTOV(vcp)->i_dentry));
-#endif
-		    dp->d_inode = AFSTOV(vcp);
-#if defined(AFS_LINUX24_ENV)
-#if defined(AFS_LINUX26_ENV)
-		    spin_unlock(&dp->d_lock);
-#endif
-#if defined(HAVE_DCACHE_LOCK)
-		    spin_unlock(&dcache_lock);
-#else
-		    spin_unlock(&AFSTOV(vcp)->i_lock);
-#endif
-#endif
-		    dput(dp);
-
-		    AFS_FAST_RELE(afs_globalVp);
-		    afs_globalVp = vcp;
-		out:
-		    crfree(credp);
-		    afs_DestroyReq(treq);
-		}
-#else
-#ifdef AFS_DARWIN80_ENV
+# ifdef AFS_DARWIN80_ENV
 		afs_PutVCache(afs_globalVp);
-#else
+# else
 		AFS_FAST_RELE(afs_globalVp);
-#endif
+# endif
 		afs_globalVp = 0;
 #endif
 	    }
@@ -583,7 +514,6 @@ BStore(struct brequest *ab)
     AFS_STATCNT(BStore);
     if ((code = afs_CreateReq(&treq, ab->cred)))
 	return;
-    code = 0;
     tvc = ab->vc;
 #if defined(AFS_SGI_ENV)
     /*
@@ -619,7 +549,7 @@ BStore(struct brequest *ab)
 	 * can know the "raw" value for interpreting the value internally, as
 	 * well as the afs_CheckCode value to give to the OS. */
 	ab->code_raw = code;
-	ab->code_checkcode = afs_CheckCode(code, treq, 43);
+	ab->code_checkcode = afs_CheckCode(code, treq, 430);
 
 	ab->flags |= BUVALID;
 	if (ab->flags & BUWAIT) {
@@ -630,6 +560,64 @@ BStore(struct brequest *ab)
     afs_DestroyReq(treq);
 }
 
+static void
+BPartialStore(struct brequest *ab)
+{
+    struct vcache *tvc;
+    afs_int32 code;
+    struct vrequest *treq = NULL;
+    int locked, shared_locked = 0;
+
+    AFS_STATCNT(BStore);
+    if ((code = afs_CreateReq(&treq, ab->cred)))
+	return;
+    tvc = ab->vc;
+    locked = tvc->lock.excl_locked? 1:0;
+    if (!locked)
+        ObtainWriteLock(&tvc->lock, 1209);
+    else if (!(tvc->lock.excl_locked & WRITE_LOCK)) {
+	shared_locked = 1;
+	ConvertSToRLock(&tvc->lock);
+    }
+    code = afs_StoreAllSegments(tvc, treq, AFS_ASYNC);
+    if (!locked)
+	ReleaseWriteLock(&tvc->lock);
+    else if (shared_locked)
+	ConvertSToRLock(&tvc->lock);
+    /* now set final return code, and wakeup anyone waiting */
+    if ((ab->flags & BUVALID) == 0) {
+	/* set final code, since treq doesn't go across processes */
+	ab->code_raw = code;
+	ab->code_checkcode = afs_CheckCode(code, treq, 43);
+	ab->flags |= BUVALID;
+	if (ab->flags & BUWAIT) {
+	    ab->flags &= ~BUWAIT;
+	    afs_osi_Wakeup(ab);
+	}
+    }
+    afs_DestroyReq(treq);
+}
+
+static void
+BInvalidateSegments(struct brequest *ab)
+{
+    int code;
+    struct vcache *tvc = ab->vc;
+    osi_Assert(WriteLocked(&tvc->lock));
+
+    code = afs_InvalidateAllSegments_once(tvc);
+
+    /* Set return code, and wakeup anyone waiting. */
+    if ((ab->flags & BUVALID) == 0) {
+	ab->code_raw = ab->code_checkcode = code;
+	ab->flags |= BUVALID;
+	if ((ab->flags & BUWAIT)) {
+	    ab->flags &= ~BUWAIT;
+	    afs_osi_Wakeup(ab);
+	}
+    }
+}
+
 /* release a held request buffer */
 void
 afs_BRelease(struct brequest *ab)
@@ -677,7 +665,9 @@ afs_BQueue(short aopcode, struct vcache *avc,
 	    tb->opcode = aopcode;
 	    tb->vc = avc;
 	    tb->cred = acred;
-	    crhold(tb->cred);
+	    if (tb->cred) {
+		crhold(tb->cred);
+	    }
 	    if (avc) {
 		AFS_FAST_HOLD(avc);
 	    }
@@ -1018,7 +1008,7 @@ brequest_release(struct brequest *tb)
     afs_BRelease(tb);  /* this grabs and releases afs_xbrs lock */
 }
 
-#ifdef AFS_DARWIN80_ENV
+#ifdef AFS_NEW_BKG
 int
 afs_BackgroundDaemon(struct afs_uspc_param *uspc, void *param1, void *param2)
 #else
@@ -1035,7 +1025,7 @@ afs_BackgroundDaemon(void)
 	/* Irix with "short stack" exits */
 	afs_BackgroundDaemon_once();
 
-#ifdef AFS_DARWIN80_ENV
+#ifdef AFS_NEW_BKG
     /* If it's a re-entering syscall, complete the request and release */
     if (uspc->ts > -1) {
         tb = afs_brs;
@@ -1059,7 +1049,7 @@ afs_BackgroundDaemon(void)
 #endif
         /* Otherwise it's a new one */
 	afs_nbrs++;
-#ifdef AFS_DARWIN80_ENV
+#ifdef AFS_NEW_BKG
     }
 #endif
 
@@ -1073,7 +1063,7 @@ afs_BackgroundDaemon(void)
 		afs_termState = AFSOP_STOP_RXCALLBACK;
 	    ReleaseWriteLock(&afs_xbrs);
 	    afs_osi_Wakeup(&afs_termState);
-#ifdef AFS_DARWIN80_ENV
+#ifdef AFS_NEW_BKG
 	    return -2;
 #else
 	    return;
@@ -1123,6 +1113,10 @@ afs_BackgroundDaemon(void)
                 return 0;
             }
 #endif
+	    else if (tb->opcode == BOP_PARTIAL_STORE)
+		BPartialStore(tb);
+	    else if (tb->opcode == BOP_INVALIDATE_SEGMENTS)
+		BInvalidateSegments(tb);
 	    else
 		panic("background bop");
 	    brequest_release(tb);
@@ -1137,7 +1131,7 @@ afs_BackgroundDaemon(void)
 	    afs_brsDaemons--;
 	}
     }
-#ifdef AFS_DARWIN80_ENV
+#ifdef AFS_NEW_BKG
     return -2;
 #endif
 }
@@ -1149,7 +1143,7 @@ shutdown_daemons(void)
     AFS_STATCNT(shutdown_daemons);
     if (afs_cold_shutdown) {
 	afs_brsDaemons = brsInit = 0;
-	rxepoch_checked = afs_nbrs = 0;
+	afs_nbrs = 0;
 	memset(afs_brs, 0, sizeof(afs_brs));
 	memset(&afs_xbrs, 0, sizeof(afs_lock_t));
 	afs_brsWaiters = 0;
