@@ -72,13 +72,15 @@ afs_FindDCacheByFid(struct VenusFid *afid)
     for (index = afs_dvhashTbl[i]; index != NULLIDX;) {
 	if (afs_indexUnique[index] == afid->Fid.Unique) {
 	    tdc = afs_GetValidDSlot(index);
-	    if (tdc) {
-		ReleaseReadLock(&tdc->tlock);
-		if (!FidCmp(&tdc->f.fid, afid)) {
-		    break;		/* leaving refCount high for caller */
-		}
-		afs_PutDCache(tdc);
-	    }
+	    if (!tdc) {
+                index = NULLIDX;
+                break;
+            }
+            ReleaseReadLock(&tdc->tlock);
+            if (!FidCmp(&tdc->f.fid, afid)) {
+                break;		/* leaving refCount high for caller */
+            }
+            afs_PutDCache(tdc);
 	}
 	index = afs_dvnextTbl[index];
     }
@@ -264,7 +266,7 @@ afs_GetVnodeName(struct vcache *avc, struct VenusFid *afid, char *aname,
 	parent_vc = afs_FindVCache(&parent_fid, 0, 1);
 	ReleaseSharedLock(&afs_xvcache);
 	if (!parent_vc) {
-	    return ENOENT;
+	    return ENETDOWN;
 	}
 
 	shadow_fid.Cell = parent_vc->f.fid.Cell;
@@ -293,7 +295,7 @@ afs_GetVnodeName(struct vcache *avc, struct VenusFid *afid, char *aname,
 	    code = ENOENT;
     } else {
 	/* printf("Directory dcache not found!\n"); */
-        code = ENOENT;
+        code = ENETDOWN;
     }
 
     return code;
@@ -507,7 +509,7 @@ afs_GetParentVCache(struct vcache *avc, int deleted, struct VenusFid *afid,
 
     if (afs_GetParentDirFid(avc, afid)) {
 	/* printf("afs_GetParentVCache: Couldn't find parent dir's FID.\n"); */
-	return ENOENT;
+	return ENETDOWN;
     }
 
     code = afs_GetVnodeName(avc, afid, aname, deleted);
@@ -521,7 +523,7 @@ afs_GetParentVCache(struct vcache *avc, int deleted, struct VenusFid *afid,
     ReleaseSharedLock(&afs_xvcache);
     if (!*adp) {
 	/* printf("afs_GetParentVCache: Couldn't find parent dir's vcache\n"); */
-	code = ENOENT;
+	code = ENETDOWN;
 	goto end;
     }
 
@@ -593,7 +595,7 @@ afs_ProcessOpRename(struct vcache *avc, struct vrequest *areq)
 	/* Get parent dir's FID.*/
     	if (afs_GetParentDirFid(avc, &new_pdir_fid)) {
 	    /* printf("afs_ProcessOpRename: Couldn't find new parent dir FID.\n"); */
-	    code = ENOENT;
+	    code = ENETDOWN;
 	    goto done;
         }
     }
@@ -607,7 +609,7 @@ afs_ProcessOpRename(struct vcache *avc, struct vrequest *areq)
 
     /* Send to data to server. */
     do {
-    	tc = afs_Conn(&old_pdir_fid, areq, SHARED_LOCK, &rxconn);
+	tc = afs_Conn(&old_pdir_fid, areq, SHARED_LOCK, &rxconn);
     	if (tc) {
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_RENAME);
 	    RX_AFS_GUNLOCK();
@@ -625,13 +627,13 @@ afs_ProcessOpRename(struct vcache *avc, struct vrequest *areq)
 	    code = -1;
 
     } while (afs_Analyze(tc,
-			 rxconn,
-			 code,
-			 &new_pdir_fid,
-			 areq,
-			 AFS_STATS_FS_RPCIDX_RENAME,
-			 SHARED_LOCK,
-			 NULL));
+                rxconn,
+    		code,
+		&new_pdir_fid,
+		areq,
+		AFS_STATS_FS_RPCIDX_RENAME,
+		SHARED_LOCK,
+		NULL));
 
     /* if (code) printf("afs_ProcessOpRename: server code=%u\n", code); */
 done:
@@ -688,7 +690,7 @@ afs_ProcessOpCreate(struct vcache *avc, struct vrequest *areq,
 
 	tdc = afs_GetDCache(avc, 0, areq, &offset, &tlen, 0);
 	if (!tdc) {
-	    code = ENOENT;
+	    code = ENETDOWN;
 	    goto end;
 	}
 
@@ -707,6 +709,7 @@ afs_ProcessOpCreate(struct vcache *avc, struct vrequest *areq,
 	}
 	ObtainReadLock(&tdc->lock);
 	tfile = afs_CFileOpen(&tdc->f.inode);
+        osi_Assert(tfile);
 	code = afs_CFileRead(tfile, 0, ttargetName, tlen);
 	ttargetName[tlen-1] = '\0';
 	afs_CFileClose(tfile);
@@ -731,7 +734,7 @@ afs_ProcessOpCreate(struct vcache *avc, struct vrequest *areq,
 		op = AFS_STATS_FS_RPCIDX_CREATEFILE;
 		XSTATS_START_TIME(op);
                 RX_AFS_GUNLOCK();
-                code = RXAFS_CreateFile(tc->id,
+                code = RXAFS_CreateFile(rxconn,
 					(struct AFSFid *)&tdp->f.fid.Fid,
 					tname, &InStatus,
 					(struct AFSFid *) &newFid.Fid,
@@ -858,6 +861,11 @@ afs_ProcessOpCreate(struct vcache *avc, struct vrequest *areq,
     for (index = afs_dvhashTbl[hash]; index != NULLIDX; index = hash) {
         hash = afs_dvnextTbl[index];
         tdc = afs_GetValidDSlot(index);
+        if (!tdc) {
+            ReleaseWriteLock(&afs_xdcache);
+            code = EIO;
+            goto end;
+        }
         ReleaseReadLock(&tdc->tlock);
 	if (afs_indexUnique[index] == avc->f.fid.Fid.Unique) {
             if (!FidCmp(&tdc->f.fid, &avc->f.fid)) {
@@ -878,8 +886,7 @@ afs_ProcessOpCreate(struct vcache *avc, struct vrequest *areq,
 		memcpy(&tdc->f.fid, &newFid, sizeof(struct VenusFid));
            }                   /* if fid match */
 	}                       /* if uniquifier match */
-    	if (tdc)
-	    afs_PutDCache(tdc);
+        afs_PutDCache(tdc);
     }                           /* for all dcaches in this hash bucket */
     ReleaseWriteLock(&afs_xdcache);
 
@@ -958,13 +965,13 @@ afs_ProcessOpRemove(struct vcache *avc, struct vrequest *areq)
 	    } else
 		code = -1;
 	} while (afs_Analyze(tc,
-			     rxconn,
-			     code,
-			     &pdir_fid,
-			     areq,
-			     AFS_STATS_FS_RPCIDX_REMOVEFILE,
-			     SHARED_LOCK,
-			     NULL));
+			rxconn,
+			code,
+			&pdir_fid,
+			areq,
+			AFS_STATS_FS_RPCIDX_REMOVEFILE,
+			SHARED_LOCK,
+			NULL));
 
     } else if (vType(avc) == VDIR) {
     	/* Remove dir on server. */
@@ -983,13 +990,13 @@ afs_ProcessOpRemove(struct vcache *avc, struct vrequest *areq)
 	   } else
 		code = -1;
 	} while (afs_Analyze(tc,
-			     rxconn,
-			     code,
-			     &pdir_fid,
-			     areq,
-			     AFS_STATS_FS_RPCIDX_REMOVEDIR,
-			     SHARED_LOCK,
-			     NULL));
+			rxconn,
+			code,
+			&pdir_fid,
+			areq,
+			AFS_STATS_FS_RPCIDX_REMOVEDIR,
+			SHARED_LOCK,
+			NULL));
 
     }				/* if (vType(avc) == VREG) */
 
@@ -1045,13 +1052,13 @@ afs_SendChanges(struct vcache *avc, struct vrequest *areq)
 		    code = -1;
 
 	} while (afs_Analyze(tc,
-			     rxconn,
-			     code,
-			     &avc->f.fid,
-			     areq,
-			     AFS_STATS_FS_RPCIDX_STORESTATUS,
-			     SHARED_LOCK,
-			     NULL));
+			rxconn,
+			code,
+			&avc->f.fid,
+			areq,
+			AFS_STATS_FS_RPCIDX_STORESTATUS,
+			SHARED_LOCK,
+			NULL));
 
 	}		/* if (afs_GenStoreStatus() > 0)*/
     }			/* disconnected SETATTR */
@@ -1084,13 +1091,13 @@ afs_SendChanges(struct vcache *avc, struct vrequest *areq)
 		code = -1;
 
 	} while (afs_Analyze(tc,
-			     rxconn,
-			     code,
-			     &avc->f.fid,
-			     areq,
-			     AFS_STATS_FS_RPCIDX_STOREDATA,
-			     SHARED_LOCK,
-			     NULL));
+			rxconn,
+			code,
+			&avc->f.fid,
+			areq,
+			AFS_STATS_FS_RPCIDX_STOREDATA,
+			SHARED_LOCK,
+			NULL));
 
     }			/* disconnected TRUNC | WRITE */
 
@@ -1159,7 +1166,7 @@ afs_ResyncDisconFiles(struct vrequest *areq, afs_ucred_t *acred)
   	    if (tc) {
 	    	XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_SETLOCK);
   		RX_AFS_GUNLOCK();
-  		code = RXAFS_SetLock(rxconn,
+		code = RXAFS_SetLock(rxconn,
 					(struct AFSFid *)&tvc->f.fid.Fid,
 					LockWrite,
 					&tsync);
@@ -1169,13 +1176,13 @@ afs_ResyncDisconFiles(struct vrequest *areq, afs_ucred_t *acred)
 		code = -1;
 
 	} while (afs_Analyze(tc,
-			     rxconn,
-			     code,
-			     &tvc->f.fid,
-			     areq,
-			     AFS_STATS_FS_RPCIDX_SETLOCK,
-			     SHARED_LOCK,
-			     NULL));
+			rxconn,
+			code,
+			&tvc->f.fid,
+			areq,
+			AFS_STATS_FS_RPCIDX_SETLOCK,
+			SHARED_LOCK,
+			NULL));
 
 	if (code)
 	    goto next_file;
@@ -1191,7 +1198,7 @@ afs_ResyncDisconFiles(struct vrequest *areq, afs_ucred_t *acred)
 	do {
 	    tc = afs_Conn(&tvc->f.fid, areq, SHARED_LOCK, &rxconn);
 	    if (tc) {
-	    	tvc->callback = tc->srvr->server;
+	    	tvc->callback = tc->parent->srvr->server;
 		start = osi_Time();
 		XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_FETCHSTATUS);
 		RX_AFS_GUNLOCK();
@@ -1206,13 +1213,13 @@ afs_ResyncDisconFiles(struct vrequest *areq, afs_ucred_t *acred)
 		code = -1;
 
 	} while (afs_Analyze(tc,
-			     rxconn,
-			     code,
-			     &tvc->f.fid,
-			     areq,
-			     AFS_STATS_FS_RPCIDX_FETCHSTATUS,
-			     SHARED_LOCK,
-			     NULL));
+			rxconn,
+			code,
+			&tvc->f.fid,
+			areq,
+			AFS_STATS_FS_RPCIDX_FETCHSTATUS,
+			SHARED_LOCK,
+			NULL));
 
 	if (code) {
 	    goto unlock_srv_file;
@@ -1258,13 +1265,13 @@ unlock_srv_file:
 	    } else
 		ucode = -1;
 	} while (afs_Analyze(tc,
-			     rxconn,
-			     ucode,
-			     &tvc->f.fid,
-			     areq,
-			     AFS_STATS_FS_RPCIDX_RELEASELOCK,
-			     SHARED_LOCK,
-			     NULL));
+			rxconn,
+			ucode,
+			&tvc->f.fid,
+			areq,
+			AFS_STATS_FS_RPCIDX_RELEASELOCK,
+			SHARED_LOCK,
+			NULL));
 #endif
 next_file:
 	ObtainWriteLock(&afs_disconDirtyLock, 710);
@@ -1507,6 +1514,7 @@ afs_GenDisconStatus(struct vcache *adp, struct vcache *avc,
 		    struct VenusFid *afid, struct vattr *attrs,
 		    struct vrequest *areq, int file_type)
 {
+    afs_hyper_t zero;
     memcpy(&avc->f.fid, afid, sizeof(struct VenusFid));
     avc->f.m.Mode = attrs->va_mode;
     /* Used to do this:
@@ -1517,7 +1525,8 @@ afs_GenDisconStatus(struct vcache *adp, struct vcache *avc,
      */
     avc->f.m.Group = adp->f.m.Group;
     avc->f.m.Owner = adp->f.m.Owner;
-    hset64(avc->f.m.DataVersion, 0, 0);
+    hzero(zero);
+    afs_SetDataVersion(avc, &zero);
     avc->f.m.Length = attrs->va_size;
     avc->f.m.Date = osi_Time();
     switch(file_type) {
@@ -1537,7 +1546,7 @@ afs_GenDisconStatus(struct vcache *adp, struct vcache *avc,
 	vSetType(avc, VLNK);
 	avc->f.m.Mode |= S_IFLNK;
 	if ((avc->f.m.Mode & 0111) == 0)
-	    avc->mvstat = 1;
+	    avc->mvstat = AFS_MVSTAT_MTPT;
 	avc->f.parent.vnode = adp->f.fid.Fid.Vnode;
 	avc->f.parent.unique = adp->f.fid.Fid.Unique;
 	break;
