@@ -162,6 +162,8 @@ afs_pag_sleep(afs_ucred_t **acred)
 static int
 afs_pag_wait(afs_ucred_t **acred)
 {
+    int code = 0;
+
     if (afs_pag_sleep(acred)) {
 	if (!afs_pag_sleepcnt) {
 	    afs_warn("%s() PAG throttling triggered, pid %d... sleeping.  sleepcnt %d\n",
@@ -171,14 +173,13 @@ afs_pag_wait(afs_ucred_t **acred)
 	afs_pag_sleepcnt++;
 
 	do {
-	    /* XXX spins on EINTR */
-	    afs_osi_Wait(1000, (struct afs_osi_WaitHandle *)0, 0);
-	} while (afs_pag_sleep(acred));
+	    code = afs_osi_Wait(1000, (struct afs_osi_WaitHandle *)0, 0);
+	} while (!code && afs_pag_sleep(acred));
 
 	afs_pag_sleepcnt--;
     }
 
-    return 0;
+    return code;
 }
 
 int
@@ -186,6 +187,8 @@ int
 afs_setpag(afs_ucred_t **credpp)
 #elif	defined(AFS_FBSD_ENV)
 afs_setpag(struct thread *td, void *args)
+#elif	defined(AFS_NBSD_ENV)
+afs_setpag(afs_proc_t *p, const void *args, register_t *retval)
 #elif  defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 afs_setpag(afs_proc_t *p, void *args, int *retval)
 #else
@@ -210,7 +213,10 @@ afs_setpag(void)
 
     AFS_STATCNT(afs_setpag);
 
-    afs_pag_wait(acred);
+    code = afs_pag_wait(acred);
+    if (code) {
+	goto done;
+    }
 
 
 #if	defined(AFS_SUN5_ENV)
@@ -218,8 +224,7 @@ afs_setpag(void)
 #elif	defined(AFS_FBSD_ENV)
     code = AddPag(td, genpag(), &td->td_ucred);
 #elif   defined(AFS_NBSD40_ENV)
-    /* XXXX won't work */
-    code = AddPag(p, genpag(), (afs_ucred_t **) osi_curcred());
+    code = AddPag(p, genpag(), &p->l_proc->p_cred);
 #elif	defined(AFS_XBSD_ENV)
     code = AddPag(p, genpag(), &p->p_rcred);
 #elif	defined(AFS_AIX41_ENV)
@@ -270,6 +275,7 @@ afs_setpag(void)
     code = AddPag(genpag(), &u.u_cred);
 #endif
 
+  done:
     afs_Trace1(afs_iclSetp, CM_TRACE_SETPAG, ICL_TYPE_INT32, code);
 
 #if defined(KERNEL_HAVE_UERROR)
@@ -321,7 +327,10 @@ afs_setpag_val(int pagval)
 
     AFS_STATCNT(afs_setpag);
 
-    afs_pag_wait(acred);
+    code = afs_pag_wait(acred);
+    if (code) {
+	goto done;
+    }
 
 #if	defined(AFS_SUN5_ENV)
     code = AddPag(pagval, credpp);
@@ -371,6 +380,7 @@ afs_setpag_val(int pagval)
     code = AddPag(pagval, &u.u_cred);
 #endif
 
+  done:
     afs_Trace1(afs_iclSetp, CM_TRACE_SETPAG, ICL_TYPE_INT32, code);
 #if defined(KERNEL_HAVE_UERROR)
     if (!getuerror())
@@ -426,8 +436,6 @@ AddPag(afs_int32 aval, afs_ucred_t **credpp)
     AFS_STATCNT(AddPag);
 #if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     if ((code = setpag(p, credpp, aval, &newpag, 0)))
-#elif defined(AFS_NBSD40_ENV)
-    if ((code = setpag(p, (void *) credpp, aval, &newpag, 0)))
 #else
     if ((code = setpag(credpp, aval, &newpag, 0)))
 #endif
@@ -613,13 +621,12 @@ osi_get_group_pag(afs_ucred_t *cred)
     ngroups = crgetngroups(cred);
 #endif
 #if defined(AFS_NBSD40_ENV)
-#warning com afs_ucred_t w/magic will not work
     if (cred == NOCRED || cred == FSCRED)
       return NOPAG;
     if (osi_crngroups(cred) < 3)
       return NOPAG;
-    g0 = osi_crgroupbyid(cred, 0);
-    g1 = osi_crgroupbyid(cred, 1);
+    g0 = osi_crgroupbyid(cred, 1);
+    g1 = osi_crgroupbyid(cred, 2);
 #elif defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     if (cred == NOCRED || cred == FSCRED)
 	return NOPAG;
