@@ -52,6 +52,9 @@
 # include <sys/timeout.h>
 
 #elif defined(AFS_NBSD40_ENV)
+# if defined(AFS_NBSD50_ENV) && !defined(AFS_NBSD70_ENV)
+#  include <sys/simplelock.h>
+# endif
 # include <sys/errno.h>
 # include <sys/types.h>
 # include <sys/mount.h> /* may define MOUNT_AFS */
@@ -82,21 +85,14 @@
 # include <sys/socket.h>
 # include <sys/socketvar.h>
 # include <sys/dirent.h>
-# include <sys/user.h>
 # include <sys/kauth.h>
 # include <sys/uio.h>
 # include <sys/buf.h>
 # include <sys/stat.h>
 # include <sys/file.h>
 # include <sys/namei.h>
-# include <sys/socket.h>
-# include <sys/socketvar.h>
-# include <sys/dirent.h>
-# include <sys/user.h>
-# include <sys/kauth.h>
 # include <sys/uio.h>
 # include <sys/buf.h>
-# include <sys/stat.h>
 # include <sys/file.h>
 # include <sys/namei.h>
 # include <sys/vnode.h>
@@ -106,9 +102,7 @@
 # include <ufs/ufs/extattr.h>
 # include <ufs/ufs/ufsmount.h>
 # ifndef MLEN
-#  if 0
-#   include <sys/mbuf.h>
-#  endif /* 0 */
+#  include <sys/mbuf.h>
 #  include <net/if.h>
 # endif /* !MLEN */
 # include <sys/protosw.h>
@@ -128,6 +122,7 @@
 # include <linux/kdev_t.h>
 # include <linux/ioctl.h>
 # if defined(AFS_LINUX26_ENV)
+#  include <linux/compat.h>
 #  include <linux/backing-dev.h>
 #  include <linux/pagemap.h>
 #  include <linux/namei.h>
@@ -151,7 +146,11 @@ struct coda_inode_info {
 struct xfs_inode_info {
 };
 # endif
-# include <asm/uaccess.h>
+# if defined(HAVE_LINUX_UACCESS_H)
+#  include <linux/uaccess.h>
+# else
+#  include <asm/uaccess.h>
+# endif
 # include <linux/list.h>
 # include <linux/dcache.h>
 # include <linux/mount.h>
@@ -174,9 +173,7 @@ struct xfs_inode_info {
 #  include <linux/mutex.h>
 # endif
 # include <linux/errno.h>
-# ifdef HAVE_LINUX_COMPLETION_H
-#  include <linux/completion.h>
-# endif
+# include <linux/completion.h>
 # if defined(HAVE_LINUX_EXPORTFS_H)
 #  include <linux/exportfs.h>
 # endif
@@ -279,12 +276,9 @@ typedef unsigned short etap_event_t;
 
 # ifdef	AFS_SUN5_ENV
 #  include <sys/cmn_err.h>	/* for kernel printf() prototype */
-# endif
-
-# if	defined(AFS_SUN56_ENV)
-#  include "h/vfs.h"		/* stops SUN56 socketvar.h warnings */
-#  include "h/stropts.h"		/* stops SUN56 socketvar.h warnings */
-#  include "h/stream.h"		/* stops SUN56 socketvar.h errors */
+#  include "h/vfs.h"		/* stops SUN5 socketvar.h warnings */
+#  include "h/stropts.h"		/* stops SUN5 socketvar.h warnings */
+#  include "h/stream.h"		/* stops SUN5 socketvar.h errors */
 # endif
 
 # ifdef AFS_SUN510_ENV
@@ -413,7 +407,6 @@ MALLOC_DECLARE(M_AFS);
 #  endif
 # endif /* AFS_SUN5_ENV */
 
-# include "rpc/types.h"
 # include "rx/xdr.h"
 
 /* Miscellaneous headers */
