@@ -10,6 +10,16 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
+
+#ifdef IGNORE_SOME_GCC_WARNINGS
+# ifdef __clang__
+#  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+# else
+#  pragma GCC diagnostic warning "-Wdeprecated-declarations"
+# endif
+#endif
+
 #define VFS 1
 
 #include <afs/cmd.h>
@@ -18,25 +28,9 @@
 
 #include <assert.h>
 #include <afs/afsutil.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <signal.h>
-#include <string.h>
-#include <stdlib.h>
-#include <time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/file.h>
-#include <errno.h>
-#include <sys/time.h>
-#include <dirent.h>
 #include <sys/wait.h>
 
-
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
-
 #if defined(AFS_LINUX20_ENV)
 #include <sys/resource.h>
 #endif
@@ -77,24 +71,16 @@
 #include <sys/fstyp.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
+#include <ctype.h>
 
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-
-#include <netinet/in.h>
+#include <afs/opr.h>
 #include <afs/afs_args.h>
 #include <afs/cellconfig.h>
-#include <ctype.h>
 #include <afs/afssyscalls.h>
 #include <afs/afsutil.h>
 
 #ifdef AFS_DARWIN_ENV
 #ifdef AFS_DARWIN80_ENV
-#include <sys/ioctl.h>
 #include <sys/xattr.h>
 #endif
 #include <mach/mach.h>
@@ -150,136 +136,162 @@ afsd_set_afsd_rtpri(void)
     SET_AFSD_RTPRI();
 }
 
-#if !defined(AFS_SGI_ENV) && !defined(AFS_AIX32_ENV)
-
+#if defined(AFS_LINUX20_ENV)
 int
-afsd_call_syscall(long param1, long param2, long param3, long param4, long param5,
-	     long param6, long  param7)
+os_syscall(struct afsd_syscall_args *args)
 {
     int error;
-# ifdef AFS_LINUX20_ENV
-    long eparm[4];
     struct afsprocdata syscall_data;
+
     int fd = open(PROC_SYSCALL_FNAME, O_RDWR);
     if (fd < 0)
 	fd = open(PROC_SYSCALL_ARLA_FNAME, O_RDWR);
-    eparm[0] = param4;
-    eparm[1] = param5;
-    eparm[2] = param6;
-    eparm[3] = param7;
 
-    param4 = (long)eparm;
+    if (fd < 0)
+	return -1;
 
     syscall_data.syscall = AFSCALL_CALL;
-    syscall_data.param1 = param1;
-    syscall_data.param2 = param2;
-    syscall_data.param3 = param3;
-    syscall_data.param4 = param4;
-    if (fd > 0) {
-	error = ioctl(fd, VIOC_SYSCALL, &syscall_data);
-	close(fd);
-    } else
-# endif /* AFS_LINUX20_ENV */
-# ifdef AFS_DARWIN80_ENV
-    struct afssysargs syscall_data;
-    void *ioctldata;
-    int fd = open(SYSCALL_DEV_FNAME,O_RDWR);
-    int syscallnum;
-#ifdef AFS_DARWIN100_ENV
-    int is64 = 0;
-    struct afssysargs64 syscall64_data;
-    if (sizeof(param1) == 8) {
-	syscallnum = VIOC_SYSCALL64;
-	is64 = 1;
-	ioctldata = &syscall64_data;
-	syscall64_data.syscall = (int)AFSCALL_CALL;
-	syscall64_data.param1 = param1;
-	syscall64_data.param2 = param2;
-	syscall64_data.param3 = param3;
-	syscall64_data.param4 = param4;
-	syscall64_data.param5 = param5;
-	syscall64_data.param6 = param6;
-    } else {
-#endif
-	syscallnum = VIOC_SYSCALL;
-        ioctldata = &syscall_data;
-	syscall_data.syscall = AFSCALL_CALL;
-	syscall_data.param1 = param1;
-	syscall_data.param2 = param2;
-	syscall_data.param3 = param3;
-	syscall_data.param4 = param4;
-	syscall_data.param5 = param5;
-	syscall_data.param6 = param6;
-#ifdef AFS_DARWIN100_ENV
-    }
-#endif
-    if(fd >= 0) {
-        error = ioctl(fd, syscallnum, ioctldata);
-        close(fd);
-    } else {
-        error = -1;
-    }
-    if (!error) {
-#ifdef AFS_DARWIN100_ENV
-        if (is64)
-            error=syscall64_data.retval;
-        else
-#endif
-            error=syscall_data.retval;
-    }
-# elif defined(AFS_SUN511_ENV)
-	{
-	    int rval;
-	    rval = ioctl_sun_afs_syscall(AFSCALL_CALL, param1, param2, param3,
-	                                 param4, param5, param6, &error);
-	    if (rval) {
-		error = rval;
-	    }
-	}
-# else /* AFS_DARWIN80_ENV */
-    error =
-	syscall(AFS_SYSCALL, AFSCALL_CALL, param1, param2, param3, param4,
-		param5, param6, param7);
-# endif /* !AFS_DARWIN80_ENV */
+    syscall_data.param1 = args->syscall;
+    syscall_data.param2 = args->params[0];
+    syscall_data.param3 = args->params[1];
+    syscall_data.param4 = (long) &args->params[2];
 
-    if (afsd_debug) {
-#ifdef AFS_NBSD40_ENV
-        char *s = strerror(errno);
-        printf("SScall(%d, %d, %d)=%d (%d, %s)\n", AFS_SYSCALL, AFSCALL_CALL,
-                param1, error, errno, s);
-#else
-	printf("SScall(%d, %d, %ld)=%d ", AFS_SYSCALL, AFSCALL_CALL, param1,
-	       error);
-#endif
-    }
-
-    return (error);
-}
-#else /* !AFS_SGI_ENV && !AFS_AIX32_ENV */
-# if defined(AFS_SGI_ENV)
-int
-afsd_call_syscall(call, parm0, parm1, parm2, parm3, parm4)
-{
-
-    int error;
-
-    error = afs_syscall(call, parm0, parm1, parm2, parm3, parm4);
-    if (afsd_verbose)
-	printf("SScall(%d, %d)=%d ", call, parm0, error);
+    error = ioctl(fd, VIOC_SYSCALL, &syscall_data);
+    close(fd);
 
     return error;
 }
-# else /* AFS_SGI_ENV */
-int
-afsd_call_syscall(call, parm0, parm1, parm2, parm3, parm4, parm5, parm6)
+#elif defined(AFS_DARWIN80_ENV)
+
+# if defined(AFS_DARWIN100_ENV)
+static int
+os_syscall64(struct afsd_syscall_args *args)
 {
+    int error;
+    struct afssysargs64 syscall64_data;
+    int fd = open(SYSCALL_DEV_FNAME, O_RDWR);
 
-    return syscall(AFSCALL_CALL, call, parm0, parm1, parm2, parm3, parm4,
-		   parm5, parm6);
+    if (fd < 0)
+	return -1;
+
+    syscall64_data.syscall = (int)AFSCALL_CALL;
+    syscall64_data.param1 = args->syscall;
+    syscall64_data.param2 = args->params[0];
+    syscall64_data.param3 = args->params[1];
+    syscall64_data.param4 = args->params[2];
+    syscall64_data.param5 = args->params[3];
+    syscall64_data.param6 = args->params[4];
+
+    error = ioctl(fd, VIOC_SYSCALL64, &syscall64_data);
+    close(fd);
+
+    if (error)
+	return error;
+
+    return syscall64_data.retval;
 }
-# endif /* !AFS_SGI_ENV */
-#endif /* AFS_SGI_ENV || AFS_AIX32_ENV */
+# endif
 
+static int
+os_syscall(struct afsd_syscall_args *args)
+{
+    int error;
+    struct afssysargs syscall_data;
+    int fd;
+
+# ifdef AFS_DARWIN100_ENV
+    if (sizeof(long) == 8)
+	return os_syscall64(args);
+# endif
+
+    fd = open(SYSCALL_DEV_FNAME, O_RDWR);
+    if (fd < 0)
+	return -1;
+
+    syscall_data.syscall = AFSCALL_CALL;
+    syscall_data.param1 = (unsigned int)(uintptr_t)args->syscall;
+    syscall_data.param2 = (unsigned int)(uintptr_t)args->params[0];
+    syscall_data.param3 = (unsigned int)(uintptr_t)args->params[1];
+    syscall_data.param4 = (unsigned int)(uintptr_t)args->params[2];
+    syscall_data.param5 = (unsigned int)(uintptr_t)args->params[3];
+    syscall_data.param6 = (unsigned int)(uintptr_t)args->params[4];
+
+    error = ioctl(fd, VIOC_SYSCALL, syscall_data);
+    close(fd);
+
+    if (error)
+	return error;
+
+    return syscall_data.retval;
+}
+
+#elif defined(AFS_SUN511_ENV)
+static int
+os_syscall(struct afsd_syscall_args *args)
+{
+    int retval, error;
+
+    error = ioctl_sun_afs_syscall(AFSCALL_CALL, args->syscall,
+				 args->params[0], args->params[1],
+				 args->params[2], args->params[3],
+				 args->params[4], &retval);
+    if (error)
+	return error;
+
+    return retval;
+}
+#elif defined(AFS_SGI_ENV)
+static int
+os_syscall(struct afsd_syscall_args *args)
+{
+    return afs_syscall(args->syscall, args->params[0], args->params[1],
+		       args->params[2], args->params[3], args->params[4]);
+}
+#elif defined(AFS_AIX32_ENV)
+static int
+os_syscall(struct afsd_syscall_args *args)
+{
+    return syscall(AFSCALL_CALL, args->syscall,
+		   args->params[0], args->params[1], args->params[2],
+		   args->params[3], args->params[4], args->params[5],
+		   args->params[6]);
+}
+#else
+static int
+os_syscall(struct afsd_syscall_args *args)
+{
+    return syscall(AFS_SYSCALL, AFSCALL_CALL, args->syscall,
+		   args->params[0], args->params[1], args->params[2],
+		   args->params[3], args->params[4], args->params[5]);
+}
+#endif
+
+int
+afsd_call_syscall(struct afsd_syscall_args *args)
+{
+    int error;
+
+    error = os_syscall(args);
+
+    if (afsd_debug) {
+        const char *syscall_str;
+#if defined(AFS_SYSCALL)
+        syscall_str = AFS_STRINGIZE(AFS_SYSCALL);
+#else
+        syscall_str = "[AFS_SYSCALL]";
+#endif
+	if (error == -1) {
+	    char *s = strerror(errno);
+	    printf("SScall(%s, %d, %d)=%d (%d, %s)\n", syscall_str, AFSCALL_CALL,
+		   (int)args->params[0], error, errno, s);
+	} else {
+	    printf("SScall(%s, %d, %d)=%d\n", syscall_str, AFSCALL_CALL,
+		   (int)args->params[0], error);
+	}
+    }
+
+    return error;
+}
 
 #ifdef	AFS_AIX_ENV
 /* Special handling for AIX's afs mount operation since they require much more
@@ -294,15 +306,12 @@ aix_vmount(const char *cacheMountDir)
     int size, error;
 
     size = sizeof(struct vmount) + ROUNDUP(strlen(cacheMountDir) + 1) + 5 * 4;
-    /* Malloc the vmount structure */
-    if ((vmountp = (struct vmount *)malloc(size)) == (struct vmount *)NULL) {
+    /* Malloc and zero the vmount structure */
+    if ((vmountp = calloc(1, size)) == NULL) {
 	printf("Can't allocate space for the vmount structure (AIX)\n");
 	exit(1);
     }
 
-    /* zero out the vmount structure */
-    memset(vmountp, '\0', size);
-
     /* transfer info into the vmount structure */
     vmountp->vmt_revision = VMT_REVISION;
     vmountp->vmt_length = size;
@@ -376,37 +385,14 @@ vmountdata(struct vmount * vmtp, char *obj, char *stub, char *host,
 #ifdef	AFS_HPUX_ENV
 #define	MOUNTED_TABLE	MNT_MNTTAB
 #else
-#ifdef	AFS_SUN5_ENV
-#define	MOUNTED_TABLE	MNTTAB
-#else
 #define	MOUNTED_TABLE	MOUNTED
 #endif
-#endif
 
 static int
 HandleMTab(char *cacheMountDir)
 {
-#if (defined (AFS_SUN_ENV) || defined (AFS_HPUX_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV) || defined(AFS_LINUX20_ENV)) && !defined(AFS_SUN58_ENV)
+#if (defined (AFS_HPUX_ENV) || defined(AFS_SGI_ENV) || defined(AFS_LINUX20_ENV))
     FILE *tfilep;
-#ifdef	AFS_SUN5_ENV
-    char tbuf[16];
-    struct mnttab tmntent;
-
-    memset(&tmntent, '\0', sizeof(struct mnttab));
-    if (!(tfilep = fopen(MOUNTED_TABLE, "a+"))) {
-	printf("Can't open %s\n", MOUNTED_TABLE);
-	perror(MNTTAB);
-	exit(-1);
-    }
-    tmntent.mnt_special = "AFS";
-    tmntent.mnt_mountp = cacheMountDir;
-    tmntent.mnt_fstype = "xx";
-    tmntent.mnt_mntopts = "rw";
-    sprintf(tbuf, "%ld", (long)time((time_t *) 0));
-    tmntent.mnt_time = tbuf;
-    putmntent(tfilep, &tmntent);
-    fclose(tfilep);
-#else
 #if defined(AFS_SGI_ENV) || defined(AFS_LINUX20_ENV)
     struct mntent tmntent;
     char *dir;
@@ -466,7 +452,6 @@ HandleMTab(char *cacheMountDir)
     addmntent(tfilep, &tmntent);
     endmntent(tfilep);
 #endif /* AFS_SGI_ENV */
-#endif /* AFS_SUN5_ENV */
 #endif /* unreasonable systems */
 #ifdef AFS_DARWIN_ENV
 #ifndef AFS_DARWIN100_ENV
@@ -553,7 +538,7 @@ afsd_fork(int wait, afsd_callback_func cb, void *rock)
     } else {
 	assert(code > 0);
 	if (wait) {
-	    assert(waitpid(code, NULL, 0) != -1);
+	    opr_Verify(waitpid(code, NULL, 0) != -1);
 	}
     }
     return 0;
@@ -591,7 +576,10 @@ main(int argc, char **argv)
     afsd_init();
 
     code = afsd_parse(argc, argv);
-    if (code) {
+    if (code == CMD_HELP) {
+	return 0; /* Displaying help is not an error. */
+    }
+    if (code != 0) {
 	return -1;
     }
 
