@@ -88,6 +88,12 @@
 #define SYS_NAME_ID_x86_darwin_170       535
 #define SYS_NAME_ID_amd64_darwin_170     536
 #define SYS_NAME_ID_arm_darwin_170       537
+#define SYS_NAME_ID_x86_darwin_180       538
+#define SYS_NAME_ID_amd64_darwin_180     539
+#define SYS_NAME_ID_arm_darwin_180       540
+#define SYS_NAME_ID_x86_darwin_190       541
+#define SYS_NAME_ID_amd64_darwin_190     542
+#define SYS_NAME_ID_arm_darwin_190       543
 
 #define SYS_NAME_ID_next_mach20		 601
 #define SYS_NAME_ID_next_mach30		 602
@@ -388,6 +394,8 @@
 #define SYS_NAME_ID_arm64_linux2	4100
 #define SYS_NAME_ID_arm64_linux26	4103
 
+#define SYS_NAME_ID_ppc64le_linux26	4203
+
 #define	AFS_REALM_SZ	64
 
 /* Specifies the number of equivalent local realm names */
