@@ -13,7 +13,6 @@
 #define AFS_XBSD_ENV		1	/* {Free,Open,Net}BSD */
 
 #define AFS_NAMEI_ENV		1	/* User space interface to file system */
-#define AFS_64BIT_ENV		1
 #define AFS_64BIT_CLIENT	1
 #define AFS_64BIT_IOPS_ENV	1	/* Needed for NAMEI */
 
@@ -31,7 +30,6 @@
 
 #undef  AFS_NONFSTRANS
 #define AFS_NONFSTRANS		1
-#define AFS_VM_RDWR_ENV		1
 #define AFS_VFS_ENV		1
 #define AFS_VFSINCL_ENV		1
 
@@ -51,7 +49,6 @@
 /* Extra kernel definitions (from kdefs file) */
 #ifdef _KERNEL
 #define AFS_GLOBAL_SUNLOCK	1
-#define	AFS_SHORTGID		0	/* are group id's short? */
 
 #if	!defined(ASSEMBLER) && !defined(__LANGUAGE_ASSEMBLY__)
 enum vcexcl { NONEXCL, EXCL };
