@@ -10,113 +10,83 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-
 #ifdef KERNEL
-#if !defined(UKERNEL)
-#include "h/types.h"
-#if !defined(AFS_LINUX26_ENV)
-#include "h/param.h"
-#endif
-#ifdef	AFS_AUX_ENV
-#include "h/mmu.h"
-#include "h/seg.h"
-#include "h/sysmacros.h"
-#include "h/signal.h"
-#include "h/errno.h"
-#endif
-#include "h/time.h"
-#if defined(AFS_AIX_ENV) || defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_LINUX20_ENV)
-#include "h/errno.h"
-#else
-#if !defined(AFS_SUN5_ENV) && !defined(AFS_LINUX20_ENV)
-#include "h/kernel.h"
-#endif
-#endif
-#if	defined(AFS_SUN56_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_FBSD_ENV) || defined(AFS_DARWIN80_ENV)
-#include "afs/sysincludes.h"
-#endif
-#if !defined(AFS_SGI64_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_OBSD48_ENV)
-#include "h/user.h"
-#endif /* AFS_SGI64_ENV */
-#include "h/uio.h"
-#ifdef	AFS_OSF_ENV
-#include <sys/mount.h>
-#include <sys/vnode.h>
-#include <ufs/inode.h>
-#endif
-#if !defined(AFS_SUN5_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_HPUX110_ENV)
-#include "h/mbuf.h"
-#endif
-#ifndef AFS_LINUX20_ENV
-#include "netinet/in.h"
-#endif
+# if !defined(UKERNEL)
+#  include "h/types.h"
+#  if !defined(AFS_LINUX26_ENV)
+#   include "h/param.h"
+#  endif
+#  ifdef	AFS_AUX_ENV
+#   include "h/mmu.h"
+#   include "h/seg.h"
+#   include "h/sysmacros.h"
+#   include "h/signal.h"
+#   include "h/errno.h"
+#  endif
+#  include "h/time.h"
+#  if defined(AFS_AIX_ENV) || defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_LINUX20_ENV)
+#   include "h/errno.h"
+#  else
+#   if !defined(AFS_SUN5_ENV) && !defined(AFS_LINUX20_ENV)
+#    include "h/kernel.h"
+#   endif
+#  endif
+#  if	defined(AFS_SUN5_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_FBSD_ENV) || defined(AFS_DARWIN80_ENV)
+#   include "afs/sysincludes.h"
+#  endif
+#  if !defined(AFS_SGI64_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_OBSD48_ENV) && !defined(AFS_NBSD_ENV)
+#   include "h/user.h"
+#  endif /* AFS_SGI64_ENV */
+#  include "h/uio.h"
+#  ifdef	AFS_OSF_ENV
+#   include <sys/mount.h>
+#   include <sys/vnode.h>
+#   include <ufs/inode.h>
+#  endif
+#  if !defined(AFS_SUN5_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_HPUX110_ENV)
+#   include "h/mbuf.h"
+#  endif
+#  ifndef AFS_LINUX20_ENV
+#   include "netinet/in.h"
+#  endif
+# else /* !defined(UKERNEL) */
+#  include "afs/stds.h"
+#  include "afs/sysincludes.h"
+# endif /* !defined(UKERNEL) */
 
 /* afs_buffer.c */
 /* These are needed because afs_prototypes.h is not included here */
 struct dcache;
-extern void *DRead(struct dcache *adc, int page);
-extern void *DNew(struct dcache *adc, int page);
+struct DirBuffer;
+extern int DRead(struct dcache *adc, int page, struct DirBuffer *);
+extern int DNew(struct dcache *adc, int page, struct DirBuffer *);
 
-#else /* !defined(UKERNEL) */
-#include "afs/stds.h"
-#include "afs/sysincludes.h"
+# include "afs/afs_osi.h"
 
-/* afs_buffer.c */
-/* These are needed because afs_prototypes.h is not included here */
-extern void *DRead(afs_int32 *fid, int page);
-extern void *DNew(afs_int32 *fid, int page);
+# include "afs/dir.h"
 
-#endif /* !defined(UKERNEL) */
-#include "afs/afs_osi.h"
-
-#include "afs/dir.h"
-
-#ifdef AFS_LINUX20_ENV
-#include "h/string.h"
-#endif
-
-/* generic renaming */
-#define	NameBlobs	afs_dir_NameBlobs
-#define	GetBlob		afs_dir_GetBlob
-#define GetVerifiedBlob afs_dir_GetVerifiedBlob
-#define	Create		afs_dir_Create
-#define	Length		afs_dir_Length
-#define	Delete		afs_dir_Delete
-#define	MakeDir		afs_dir_MakeDir
-#define	Lookup		afs_dir_Lookup
-#define	LookupOffset	afs_dir_LookupOffset
-#define	EnumerateDir	afs_dir_EnumerateDir
-#define	IsEmpty		afs_dir_IsEmpty
-#define InverseLookup   afs_dir_InverseLookup
-#define ChangeFid	afs_dir_ChangeFid
+# ifdef AFS_LINUX20_ENV
+#  include "h/string.h"
+# endif
 
 #else /* KERNEL */
 
-# ifdef HAVE_UNISTD_H
-#  include <unistd.h>
-# endif
-# include <sys/types.h>
-# include <errno.h>
+# include <roken.h>
 # include "dir.h"
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#endif
-
-#include <string.h>
 #endif /* KERNEL */
 
 afs_int32 DErrno;
 
 /* Local static prototypes */
-static struct DirEntry *FindItem(void *dir, char *ename,
-				 unsigned short **previtem);
-
+static int FindBlobs(dir_file_t, int);
+static void AddPage(dir_file_t, int);
+static void FreeBlobs(dir_file_t, int, int);
+static int FindItem(dir_file_t, char *, struct DirBuffer *,
+		    struct DirBuffer *);
 
 /* Find out how many entries are required to store a name. */
 int
-NameBlobs(char *name)
+afs_dir_NameBlobs(char *name)
 {
     int i;
     i = strlen(name) + 1;
@@ -126,59 +96,72 @@ NameBlobs(char *name)
 /* Create an entry in a file.  Dir is a file representation, while entry is
  * a string name. */
 int
-Create(void *dir, char *entry, void *voidfid)
+afs_dir_Create(dir_file_t dir, char *entry, void *voidfid)
 {
     afs_int32 *vfid = (afs_int32 *) voidfid;
     int blobs, firstelt;
     int i;
+    struct DirBuffer entrybuf, prevbuf, headerbuf;
     struct DirEntry *ep;
-    unsigned short *pp = NULL;
     struct DirHeader *dhp;
+    int code;
 
     /* check name quality */
     if (*entry == 0)
 	return EINVAL;
+
     /* First check if file already exists. */
-    ep = FindItem(dir, entry, &pp);
-    if (ep) {
-	DRelease(ep, 0);
-	DRelease(pp, 0);
+    code = FindItem(dir, entry, &prevbuf, &entrybuf);
+    if (code && code != ENOENT) {
+        return code;
+    }
+    if (code == 0) {
+	DRelease(&entrybuf, 0);
+	DRelease(&prevbuf, 0);
 	return EEXIST;
     }
-    blobs = NameBlobs(entry);	/* number of entries required */
+
+    blobs = afs_dir_NameBlobs(entry);	/* number of entries required */
     firstelt = FindBlobs(dir, blobs);
     if (firstelt < 0)
 	return EFBIG;		/* directory is full */
+
     /* First, we fill in the directory entry. */
-    ep = GetBlob(dir, firstelt);
-    if (ep == 0)
+    if (afs_dir_GetBlob(dir, firstelt, &entrybuf) != 0)
 	return EIO;
+    ep = (struct DirEntry *)entrybuf.data;
+
     ep->flag = FFIRST;
     ep->fid.vnode = htonl(vfid[1]);
     ep->fid.vunique = htonl(vfid[2]);
     strcpy(ep->name, entry);
+
     /* Now we just have to thread it on the hash table list. */
-    dhp = (struct DirHeader *)DRead(dir, 0);
-    if (!dhp) {
-	DRelease(ep, 1);
+    if (DRead(dir, 0, &headerbuf) != 0) {
+	DRelease(&entrybuf, 1);
 	return EIO;
     }
-    i = DirHash(entry);
+    dhp = (struct DirHeader *)headerbuf.data;
+
+    i = afs_dir_DirHash(entry);
     ep->next = dhp->hashTable[i];
     dhp->hashTable[i] = htons(firstelt);
-    DRelease(dhp, 1);
-    DRelease(ep, 1);
+    DRelease(&headerbuf, 1);
+    DRelease(&entrybuf, 1);
     return 0;
 }
 
 int
-Length(void *dir)
+afs_dir_Length(dir_file_t dir)
 {
     int i, ctr;
+    struct DirBuffer headerbuf;
     struct DirHeader *dhp;
-    dhp = (struct DirHeader *)DRead(dir, 0);
-    if (!dhp)
+
+    if (DRead(dir, 0, &headerbuf) != 0)
 	return 0;
+    dhp = (struct DirHeader *)headerbuf.data;
+
     if (dhp->header.pgcount != 0)
 	ctr = ntohs(dhp->header.pgcount);
     else {
@@ -188,44 +171,57 @@ Length(void *dir)
 	    if (dhp->alloMap[i] != EPP)
 		ctr++;
     }
-    DRelease(dhp, 0);
+    DRelease(&headerbuf, 0);
     return ctr * AFS_PAGESIZE;
 }
 
+/* Delete an entry from a directory, including update of all free entry
+ * descriptors. */
 int
-Delete(void *dir, char *entry)
+afs_dir_Delete(dir_file_t dir, char *entry)
 {
-    /* Delete an entry from a directory, including update of all free entry descriptors. */
+
     int nitems, index;
+    struct DirBuffer entrybuf, prevbuf;
     struct DirEntry *firstitem;
     unsigned short *previtem;
-    firstitem = FindItem(dir, entry, &previtem);
-    if (firstitem == 0)
-	return ENOENT;
+    int code;
+
+    code = FindItem(dir, entry, &prevbuf, &entrybuf);
+    if (code) {
+        return code;
+    }
+
+    firstitem = (struct DirEntry *)entrybuf.data;
+    previtem = (unsigned short *)prevbuf.data;
+
     *previtem = firstitem->next;
-    DRelease(previtem, 1);
-    index = DVOffset(firstitem) / 32;
-    nitems = NameBlobs(firstitem->name);
+    DRelease(&prevbuf, 1);
+    index = DVOffset(&entrybuf) / 32;
+    nitems = afs_dir_NameBlobs(firstitem->name);
     /* Clear entire DirEntry and any DirXEntry extensions */
     memset(firstitem, 0, nitems * sizeof(*firstitem));
-    DRelease(firstitem, 1);
+    DRelease(&entrybuf, 1);
     FreeBlobs(dir, index, nitems);
     return 0;
 }
 
-int
-FindBlobs(void *dir, int nblobs)
+/* Find a bunch of contiguous entries; at least nblobs in a row. */
+static int
+FindBlobs(dir_file_t dir, int nblobs)
 {
-    /* Find a bunch of contiguous entries; at least nblobs in a row. */
     int i, j, k;
     int failed = 0;
+    struct DirBuffer headerbuf, pagebuf;
     struct DirHeader *dhp;
     struct PageHeader *pp;
     int pgcount;
 
-    dhp = (struct DirHeader *)DRead(dir, 0);	/* read the dir header in first. */
-    if (!dhp)
+    /* read the dir header in first. */
+    if (DRead(dir, 0, &headerbuf) != 0)
 	return -1;
+    dhp = (struct DirHeader *)headerbuf.data;
+
     for (i = 0; i < BIGMAXPAGES; i++) {
 	if (i >= MAXPAGES || dhp->alloMap[i] >= nblobs) {
 	    /* if page could contain enough entries */
@@ -248,11 +244,12 @@ FindBlobs(void *dir, int nblobs)
 		dhp->alloMap[i] = EPP - 1;
 		dhp->header.pgcount = htons(i + 1);
 	    }
-	    pp = (struct PageHeader *)DRead(dir, i);	/* read the page in. */
-	    if (!pp) {
-		DRelease(dhp, 1);
+
+	    /* read the page in. */
+	    if (DRead(dir, i, &pagebuf) != 0) {
 		break;
 	    }
+	    pp = (struct PageHeader *)pagebuf.data;
 	    for (j = 0; j <= EPP - nblobs; j++) {
 		failed = 0;
 		for (k = 0; k < nblobs; k++)
@@ -269,27 +266,31 @@ FindBlobs(void *dir, int nblobs)
 		 * and free up any resources we've got allocated. */
 		if (i < MAXPAGES)
 		    dhp->alloMap[i] -= nblobs;
-		DRelease(dhp, 1);
+		DRelease(&headerbuf, 1);
 		for (k = 0; k < nblobs; k++)
 		    pp->freebitmap[(j + k) >> 3] |= 1 << ((j + k) & 7);
-		DRelease(pp, 1);
+		DRelease(&pagebuf, 1);
 		return j + i * EPP;
 	    }
-	    DRelease(pp, 0);	/* This dir page is unchanged. */
+	    DRelease(&pagebuf, 0);	/* This dir page is unchanged. */
 	}
     }
     /* If we make it here, the directory is full. */
-    DRelease(dhp, 1);
+    DRelease(&headerbuf, 1);
     return -1;
 }
 
-void
-AddPage(void *dir, int pageno)
+static void
+AddPage(dir_file_t dir, int pageno)
 {				/* Add a page to a directory. */
     int i;
     struct PageHeader *pp;
+    struct DirBuffer pagebuf;
+
+    /* Get a new buffer labelled dir,pageno */
+    DNew(dir, pageno, &pagebuf);
+    pp = (struct PageHeader *)pagebuf.data;
 
-    pp = (struct PageHeader *)DNew(dir, pageno);	/* Get a new buffer labelled dir,pageno */
     pp->tag = htons(1234);
     if (pageno > 0)
 	pp->pgcount = 0;
@@ -297,32 +298,39 @@ AddPage(void *dir, int pageno)
     pp->freebitmap[0] = 0x01;
     for (i = 1; i < EPP / 8; i++)	/* It's a constant */
 	pp->freebitmap[i] = 0;
-    DRelease(pp, 1);
+    DRelease(&pagebuf, 1);
 }
 
 /* Free a whole bunch of directory entries. */
 
-void
-FreeBlobs(void *dir, int firstblob, int nblobs)
+static void
+FreeBlobs(dir_file_t dir, int firstblob, int nblobs)
 {
     int i;
     int page;
+    struct DirBuffer headerbuf, pagehdbuf;
     struct DirHeader *dhp;
     struct PageHeader *pp;
     page = firstblob / EPP;
     firstblob -= EPP * page;	/* convert to page-relative entry */
-    dhp = (struct DirHeader *)DRead(dir, 0);
-    if (!dhp)
+
+    if (DRead(dir, 0, &headerbuf) != 0)
 	return;
+    dhp = (struct DirHeader *)headerbuf.data;
+
     if (page < MAXPAGES)
 	dhp->alloMap[page] += nblobs;
-    DRelease(dhp, 1);
-    pp = (struct PageHeader *)DRead(dir, page);
-    if (pp)
-	for (i = 0; i < nblobs; i++)
-	    pp->freebitmap[(firstblob + i) >> 3] &=
-		~(1 << ((firstblob + i) & 7));
-    DRelease(pp, 1);
+
+    DRelease(&headerbuf, 1);
+
+    if (DRead(dir, page, &pagehdbuf) != 0)
+	return;
+    pp = (struct PageHeader *)pagehdbuf.data;
+
+    for (i = 0; i < nblobs; i++)
+	pp->freebitmap[(firstblob + i) >> 3] &= ~(1 << ((firstblob + i) & 7));
+
+    DRelease(&pagehdbuf, 1);
 }
 
 /*
@@ -331,11 +339,15 @@ FreeBlobs(void *dir, int firstblob, int nblobs)
  * allocation map and 8 to the hash table.
  */
 int
-MakeDir(void *dir, afs_int32 * me, afs_int32 * parent)
+afs_dir_MakeDir(dir_file_t dir, afs_int32 * me, afs_int32 * parent)
 {
     int i;
+    struct DirBuffer buffer;
     struct DirHeader *dhp;
-    dhp = (struct DirHeader *)DNew(dir, 0);
+
+    DNew(dir, 0, &buffer);
+    dhp = (struct DirHeader *)buffer.data;
+
     dhp->header.pgcount = htons(1);
     dhp->header.tag = htons(1234);
     dhp->header.freecount = (EPP - DHE - 1);
@@ -348,70 +360,83 @@ MakeDir(void *dir, afs_int32 * me, afs_int32 * parent)
 	dhp->alloMap[i] = EPP;
     for (i = 0; i < NHASHENT; i++)
 	dhp->hashTable[i] = 0;
-    DRelease(dhp, 1);
-    Create(dir, ".", me);
-    Create(dir, "..", parent);	/* Virtue is its own .. */
+    DRelease(&buffer, 1);
+    afs_dir_Create(dir, ".", me);
+    afs_dir_Create(dir, "..", parent);	/* Virtue is its own .. */
     return 0;
 }
 
 /* Look up a file name in directory. */
 
 int
-Lookup(void *dir, char *entry, void *voidfid)
+afs_dir_Lookup(dir_file_t dir, char *entry, void *voidfid)
 {
     afs_int32 *fid = (afs_int32 *) voidfid;
+    struct DirBuffer firstbuf, prevbuf;
     struct DirEntry *firstitem;
-    unsigned short *previtem;
+    int code;
+
+    code = FindItem(dir, entry, &prevbuf, &firstbuf);
+    if (code) {
+        return code;
+    }
+    DRelease(&prevbuf, 0);
+    firstitem = (struct DirEntry *)firstbuf.data;
 
-    firstitem = FindItem(dir, entry, &previtem);
-    if (firstitem == 0)
-	return ENOENT;
-    DRelease(previtem, 0);
     fid[1] = ntohl(firstitem->fid.vnode);
     fid[2] = ntohl(firstitem->fid.vunique);
-    DRelease(firstitem, 0);
+    DRelease(&firstbuf, 0);
     return 0;
 }
 
 /* Look up a file name in directory. */
 
 int
-LookupOffset(void *dir, char *entry, void *voidfid, long *offsetp)
+afs_dir_LookupOffset(dir_file_t dir, char *entry, void *voidfid,
+		     long *offsetp)
 {
     afs_int32 *fid = (afs_int32 *) voidfid;
+    struct DirBuffer firstbuf, prevbuf;
     struct DirEntry *firstitem;
-    unsigned short *previtem;
+    int code;
+
+    code = FindItem(dir, entry, &prevbuf, &firstbuf);
+    if (code) {
+        return code;
+    }
+    DRelease(&prevbuf, 0);
+    firstitem = (struct DirEntry *)firstbuf.data;
 
-    firstitem = FindItem(dir, entry, &previtem);
-    if (firstitem == 0)
-	return ENOENT;
-    DRelease(previtem, 0);
     fid[1] = ntohl(firstitem->fid.vnode);
     fid[2] = ntohl(firstitem->fid.vunique);
     if (offsetp)
-	*offsetp = DVOffset(firstitem);
-    DRelease(firstitem, 0);
+	*offsetp = DVOffset(&firstbuf);
+    DRelease(&firstbuf, 0);
     return 0;
 }
 
+/*
+ * Enumerate the contents of a directory. Break when hook function
+ * returns non 0.
+ */
+
 int
-EnumerateDir(void *dir, int (*hookproc) (void *dir, char *name,
-				         afs_int32 vnode, afs_int32 unique),
-	     void *hook)
+afs_dir_EnumerateDir(dir_file_t dir, int (*proc) (void *, char *name,
+						  afs_int32 vnode,
+						  afs_int32 unique),
+		     void *hook)
 {
-    /* Enumerate the contents of a directory.
-     * Break when hook function returns non 0.
-     */
     int i;
     int num;
+    struct DirBuffer headerbuf, entrybuf;
     struct DirHeader *dhp;
     struct DirEntry *ep;
     int code = 0;
     int elements;
 
-    dhp = (struct DirHeader *)DRead(dir, 0);
-    if (!dhp)
-	return EIO;		/* first page should be there */
+    if (DRead(dir, 0, &headerbuf) != 0)
+	return EIO;
+    dhp = (struct DirHeader *)headerbuf.data;
 
     for (i = 0; i < NHASHENT; i++) {
 	/* For each hash chain, enumerate everyone on the list. */
@@ -421,41 +446,45 @@ EnumerateDir(void *dir, int (*hookproc) (void *dir, char *name,
 	    elements++;
 
 	    /* Walk down the hash table list. */
-	    code = GetVerifiedBlob(dir, num, &ep);
+	    code = afs_dir_GetVerifiedBlob(dir, num, &entrybuf);
 	    if (code)
 		goto out;
 
-	    if (!ep)
+	    ep = (struct DirEntry *)entrybuf.data;
+	    if (!ep) {
+		DRelease(&entrybuf, 0);
 		break;
+	    }
 
 	    num = ntohs(ep->next);
-	    code = (*hookproc) (hook, ep->name, ntohl(ep->fid.vnode),
-			 ntohl(ep->fid.vunique));
-	    DRelease(ep, 0);
+	    code = (*proc) (hook, ep->name, ntohl(ep->fid.vnode),
+			    ntohl(ep->fid.vunique));
+	    DRelease(&entrybuf, 0);
 	    if (code)
 		goto out;
 	}
     }
 
 out:
-    DRelease(dhp, 0);
+    DRelease(&headerbuf, 0);
     return 0;
 }
 
 int
-IsEmpty(void *dir)
+afs_dir_IsEmpty(dir_file_t dir)
 {
     /* Enumerate the contents of a directory. */
     int i;
     int num;
+    struct DirBuffer headerbuf, entrybuf;
     struct DirHeader *dhp;
     struct DirEntry *ep;
-    int code;
     int elements;
 
-    dhp = (struct DirHeader *)DRead(dir, 0);
-    if (!dhp)
+    if (DRead(dir, 0, &headerbuf) != 0)
 	return 0;
+    dhp = (struct DirHeader *)headerbuf.data;
+
     for (i = 0; i < NHASHENT; i++) {
 	/* For each hash chain, enumerate everyone on the list. */
 	num = ntohs(dhp->hashTable[i]);
@@ -463,25 +492,19 @@ IsEmpty(void *dir)
 	while (num != 0 && elements < BIGMAXPAGES * EPP) {
 	    elements++;
 	    /* Walk down the hash table list. */
-	    code = GetVerifiedBlob(dir, num, &ep);
-	    if (code)
-		goto out;
-
-	    if (!ep)
-		break;
-
+	    if (afs_dir_GetVerifiedBlob(dir, num, &entrybuf) != 0)
+	        break;
+	    ep = (struct DirEntry *)entrybuf.data;
 	    if (strcmp(ep->name, "..") && strcmp(ep->name, ".")) {
-		DRelease(ep, 0);
-		DRelease(dhp, 0);
+		DRelease(&entrybuf, 0);
+		DRelease(&headerbuf, 0);
 		return 1;
 	    }
 	    num = ntohs(ep->next);
-	    DRelease(ep, 0);
+	    DRelease(&entrybuf, 0);
 	}
     }
-
-out:
-    DRelease(dhp, 0);
+    DRelease(&headerbuf, 0);
     return 0;
 }
 
@@ -490,31 +513,35 @@ out:
  * page.
  */
 
-static struct DirEntry *
-GetBlobWithLimit(void *dir, afs_int32 blobno, afs_size_t *maxlen)
+static int
+GetBlobWithLimit(dir_file_t dir, afs_int32 blobno,
+	         struct DirBuffer *buffer, afs_size_t *maxlen)
 {
-    char *page;
     afs_size_t pos;
+    int code;
 
     *maxlen = 0;
+    memset(buffer, 0, sizeof(struct DirBuffer));
 
-    page = DRead(dir, blobno >> LEPP);
-    if (!page)
-	return NULL;
+    code = DRead(dir, blobno >> LEPP, buffer);
+    if (code)
+	return code;
 
     pos = 32 * (blobno & (EPP - 1));
 
     *maxlen = AFS_PAGESIZE - pos - 1;
 
-    return (struct DirEntry *)(page + pos);
+    buffer->data = (void *)(((char *)buffer->data) + pos);
+
+    return 0;
 }
 
 /* Given an entries number, return a pointer to that entry */
-struct DirEntry *
-GetBlob(void *dir, afs_int32 blobno)
+int
+afs_dir_GetBlob(dir_file_t dir, afs_int32 blobno, struct DirBuffer *buffer)
 {
     afs_size_t maxlen = 0;
-    return GetBlobWithLimit(dir, blobno, &maxlen);
+    return GetBlobWithLimit(dir, blobno, buffer, &maxlen);
 }
 
 /* Return an entry, having verified that the name held within the entry
@@ -523,34 +550,36 @@ GetBlob(void *dir, afs_int32 blobno)
  */
 
 int
-GetVerifiedBlob(void *file, afs_int32 blobno, struct DirEntry **outdir)
+afs_dir_GetVerifiedBlob(dir_file_t file, afs_int32 blobno,
+			struct DirBuffer *outbuf)
 {
     struct DirEntry *dir;
+    struct DirBuffer buffer;
     afs_size_t maxlen;
+    int code;
     char *cp;
 
-    *outdir = NULL;
+    code = GetBlobWithLimit(file, blobno, &buffer, &maxlen);
+    if (code)
+	return code;
 
-    DErrno = 0;
-    dir = GetBlobWithLimit(file, blobno, &maxlen);
-    if (!dir)
-	return DErrno;
+    dir = (struct DirEntry *)buffer.data;
 
     /* A blob is only valid if the name within it is NULL terminated before
      * the end of the blob's containing page */
     for (cp = dir->name; *cp != '\0' && cp < ((char *)dir) + maxlen; cp++);
 
     if (*cp != '\0') {
-	DRelease(dir, 0);
+	DRelease(&buffer, 0);
 	return EIO;
     }
 
-    *outdir = dir;
+    *outbuf = buffer;
     return 0;
 }
 
 int
-DirHash(char *string)
+afs_dir_DirHash(char *string)
 {
     /* Hash a string to a number between 0 and NHASHENT. */
     unsigned char tc;
@@ -564,7 +593,7 @@ DirHash(char *string)
     tval = hval & (NHASHENT - 1);
     if (tval == 0)
 	return tval;
-    else if (hval >= 1<<31)
+    else if (hval >= 1u<<31)
 	tval = NHASHENT - tval;
     return tval;
 }
@@ -576,117 +605,162 @@ DirHash(char *string)
  * found, however, no items are left locked, and a null pointer is
  * returned instead. */
 
-static struct DirEntry *
-FindItem(void *dir, char *ename, unsigned short **previtem)
+static int
+FindItem(dir_file_t dir, char *ename, struct DirBuffer *prevbuf,
+         struct DirBuffer *itembuf )
 {
-    int i;
+    int i, code;
+    struct DirBuffer curr, prev;
     struct DirHeader *dhp;
-    unsigned short *lp;
     struct DirEntry *tp;
     int elements;
 
-    i = DirHash(ename);
-    dhp = (struct DirHeader *)DRead(dir, 0);
-    if (!dhp)
-	return 0;
+    memset(prevbuf, 0, sizeof(struct DirBuffer));
+    memset(itembuf, 0, sizeof(struct DirBuffer));
+
+    code = DRead(dir, 0, &prev);
+    if (code)
+	return code;
+    dhp = (struct DirHeader *)prev.data;
+
+    i = afs_dir_DirHash(ename);
     if (dhp->hashTable[i] == 0) {
 	/* no such entry */
-	DRelease(dhp, 0);
-	return 0;
+	code = ENOENT;
+	goto out;
     }
 
-    GetVerifiedBlob(dir, (u_short) ntohs(dhp->hashTable[i]), &tp);
-    if (tp == NULL) {
-	DRelease(dhp, 0);
-	return NULL;
+    code = afs_dir_GetVerifiedBlob(dir,
+				   (u_short) ntohs(dhp->hashTable[i]),
+				   &curr);
+    if (code) {
+	goto out;
     }
 
-    lp = &(dhp->hashTable[i]);
+    prev.data = &(dhp->hashTable[i]);
     elements = 0;
     /* Detect circular hash chains. Absolute max size of a directory */
     while (elements < BIGMAXPAGES * EPP) {
 	elements++;
 
-	/* Look at each hash conflict entry. */
+	/* Look at each entry on the hash chain */
+	tp = (struct DirEntry *)curr.data;
 	if (!strcmp(ename, tp->name)) {
-	    /* Found our entry. */
-	    *previtem = lp;
-	    return tp;
+	    /* Found it! */
+	    *prevbuf = prev;
+	    *itembuf = curr;
+	    return 0;
 	}
 
-	DRelease(lp, 0);
-	lp = &(tp->next);
-	if (tp->next == 0)
-	    break;
+	DRelease(&prev, 0);
 
-	GetVerifiedBlob(dir, (u_short) ntohs(tp->next), &tp);
-	if (tp == NULL)
-	    break;
+	prev = curr;
+	prev.data = &(tp->next);
+
+	if (tp->next == 0) {
+	    /* The end of the line */
+	    code = ENOENT;
+	    goto out;
+	}
+
+	code = afs_dir_GetVerifiedBlob(dir, (u_short) ntohs(tp->next),
+				       &curr);
+	if (code)
+	    goto out;
     }
-    DRelease(lp, 0);
-    return NULL;
+
+    /* If we've reached here, we've hit our loop limit. Something is weird with
+     * the directory; maybe a circular hash chain? */
+    code = EIO;
+
+out:
+    DRelease(&prev, 0);
+    return code;
 }
 
-static struct DirEntry *
-FindFid (void *dir, afs_uint32 vnode, afs_uint32 unique)
+static int
+FindFid (void *dir, afs_uint32 vnode, afs_uint32 unique,
+	 struct DirBuffer *itembuf)
 {
     /* Find a directory entry, given the vnode and uniquifier of a object.
      * This entry returns a pointer to a locked buffer.  If no entry is found,
      * however, no items are left locked, and a null pointer is returned
      * instead.
      */
-    int i;
+    int i, code;
+    unsigned short next;
+    struct DirBuffer curr, header;
     struct DirHeader *dhp;
-    unsigned short *lp;
     struct DirEntry *tp;
     int elements;
 
-    dhp = (struct DirHeader *) DRead(dir,0);
-    if (!dhp) return 0;
+    memset(itembuf, 0, sizeof(struct DirBuffer));
+
+    code = DRead(dir, 0, &header);
+    if (code)
+	return code;
+    dhp = (struct DirHeader *)header.data;
+
     for (i=0; i<NHASHENT; i++) {
 	if (dhp->hashTable[i] != 0) {
-	    GetVerifiedBlob(dir,(u_short)ntohs(dhp->hashTable[i]), &tp);
-	    if (!tp) { /* should not happen */
-		DRelease(dhp, 0);
-		return 0;
+	    code = afs_dir_GetVerifiedBlob(dir,
+					   (u_short)ntohs(dhp->hashTable[i]),
+					   &curr);
+	    if (code) {
+		DRelease(&header, 0);
+		return code;
 	    }
 	    elements = 0;
-	    while(tp && elements < BIGMAXPAGES * EPP) {
+	    while(curr.data != NULL && elements < BIGMAXPAGES * EPP) {
 		elements++;
+		tp = (struct DirEntry *)curr.data;
+
 		if (vnode == ntohl(tp->fid.vnode)
 		    && unique == ntohl(tp->fid.vunique)) {
-		    DRelease(dhp, 0);
-		    return tp;
+		    DRelease(&header, 0);
+		    *itembuf = curr;
+		    return 0;
 		}
-		lp = &(tp->next);
-		if (tp->next == 0)
+
+		next = tp->next;
+		DRelease(&curr, 0);
+
+		if (next == 0)
 		    break;
-		GetVerifiedBlob(dir,(u_short)ntohs(tp->next), &tp);
-		DRelease(lp, 0);
+
+		code = afs_dir_GetVerifiedBlob(dir, (u_short)ntohs(next),
+					       &curr);
+		if (code) {
+		    DRelease(&header, 0);
+		    return code;
+		}
 	    }
-	    DRelease(lp, 0);
 	}
     }
-    DRelease(dhp, 0);
-    return NULL;
+    DRelease(&header, 0);
+    return ENOENT;
 }
 
 int
-InverseLookup (void *dir, afs_uint32 vnode, afs_uint32 unique, char *name,
-	       afs_uint32 length)
+afs_dir_InverseLookup(void *dir, afs_uint32 vnode, afs_uint32 unique,
+		      char *name, afs_uint32 length)
 {
     /* Look for the name pointing to given vnode and unique in a directory */
+    struct DirBuffer entrybuf;
     struct DirEntry *entry;
     int code = 0;
 
-    entry = FindFid(dir, vnode, unique);
-    if (!entry)
-	return ENOENT;
+    code = FindFid(dir, vnode, unique, &entrybuf);
+    if (code) {
+        return code;
+    }
+    entry = (struct DirEntry *)entrybuf.data;
+
     if (strlen(entry->name) >= length)
 	code = E2BIG;
     else
 	strcpy(name, entry->name);
-    DRelease(entry, 0);
+    DRelease(&entrybuf, 0);
     return code;
 }
 
@@ -699,22 +773,24 @@ InverseLookup (void *dir, afs_uint32 vnode, afs_uint32 unique, char *name,
  * It can be omitted if you don't need a safety check...
  * \param new_fid The new find in MKFid format (host order).
  */
-int ChangeFid(void *dir,
-		char *entry,
-		afs_uint32 *old_fid,
-		afs_uint32 *new_fid)
+int
+afs_dir_ChangeFid(dir_file_t dir, char *entry, afs_uint32 *old_fid,
+		  afs_uint32 *new_fid)
 {
+    struct DirBuffer prevbuf, entrybuf;
     struct DirEntry *firstitem;
-    unsigned short *previtem;
     struct MKFid *fid_old = (struct MKFid *) old_fid;
     struct MKFid *fid_new = (struct MKFid *) new_fid;
+    int code;
 
     /* Find entry. */
-    firstitem = FindItem(dir, entry, &previtem);
-    if (firstitem == 0) {
-	return ENOENT;
+    code = FindItem(dir, entry, &prevbuf, &entrybuf);
+    if (code) {
+        return code;
     }
-    DRelease(previtem, 1);
+    firstitem = (struct DirEntry *)entrybuf.data;
+    DRelease(&prevbuf, 1);
+
     /* Replace fid. */
     if (!old_fid ||
     	((htonl(fid_old->vnode) == firstitem->fid.vnode) &&
@@ -724,7 +800,7 @@ int ChangeFid(void *dir,
 	firstitem->fid.vunique = htonl(fid_new->vunique);
     }
 
-    DRelease(firstitem, 1);
+    DRelease(&entrybuf, 1);
 
     return 0;
 }
