@@ -16,21 +16,19 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-
-#include <string.h>
-
-#include "fsprobe.h"		/*Interface for this module */
 #include <lwp.h>		/*Lightweight process package */
 #include <afs/cellconfig.h>
 #include <afs/afsint.h>
 #include <afs/afsutil.h>
 #include <afs/volser.h>
 #include <afs/volser_prototypes.h>
-
 #define FSINT_COMMON_XG
 #include <afs/afscbint.h>
 
+#include "fsprobe.h"		/*Interface for this module */
+
 #define LWP_STACK_SIZE	(16 * 1024)
 
 /*
@@ -225,8 +223,8 @@ fsprobe_LWP(void *unused)
     struct ProbeViceStatistics *curr_stats;	/*Current stats region */
     int *curr_probeOK;		/*Current probeOK field */
     ViceStatistics64 stats64;      /*Current stats region */
-    stats64.ViceStatistics64_val = (afs_uint64 *)malloc(STATS64_VERSION *
-							sizeof(afs_uint64));
+    stats64.ViceStatistics64_val = malloc(STATS64_VERSION *
+					  sizeof(afs_uint64));
     while (1) {			/*Service loop */
 	/*
 	 * Iterate through the server connections, gathering data.
@@ -285,7 +283,7 @@ fsprobe_LWP(void *unused)
 		char pname[10];
 		struct diskPartition partition;
 		struct diskPartition64 *partition64p =
-		    (struct diskPartition64 *)malloc(sizeof(struct diskPartition64));
+		    malloc(sizeof(struct diskPartition64));
 
 		if (fsprobe_debug)
 		    fprintf(stderr,
@@ -369,9 +367,8 @@ fsprobe_LWP(void *unused)
 	if (code)
 	    fprintf(stderr, "[%s] IOMGR_Select returned code %d\n", rn, code);
     }				/*Service loop */
-    free(stats64.ViceStatistics64_val);
-
-    return NULL;
+    AFS_UNREACHED(free(stats64.ViceStatistics64_val));
+    AFS_UNREACHED(return(NULL));
 }				/*fsprobe_LWP */
 
 /*list all the partitions on <aserver> */
@@ -512,7 +509,7 @@ fsprobe_Init(int a_numServers, struct sockaddr_in *a_socketArray,
 		a_ProbeFreqInSecs);
 	arg_errfound = 1;
     }
-    if (a_ProbeHandler == (int (*)())0) {
+    if (a_ProbeHandler == NULL) {
 	fprintf(stderr, "[%s] Null probe handler function argument\n", rn);
 	arg_errfound = 1;
     }
@@ -566,7 +563,7 @@ fsprobe_Init(int a_numServers, struct sockaddr_in *a_socketArray,
 		rn, fsprobe_statsBytes);
 
     fsprobe_probeOKBytes = a_numServers * sizeof(int);
-    fsprobe_Results.probeOK = (int *)malloc(fsprobe_probeOKBytes);
+    fsprobe_Results.probeOK = malloc(fsprobe_probeOKBytes);
     if (fsprobe_Results.probeOK == (int *)0) {
 	fprintf(stderr,
 		"[%s] Can't allocate %d probeOK array entries (%d bytes)\n",
