@@ -7,14 +7,18 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#define UBIK_LEGACY_CALLITER 1
+
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
-#include <stdio.h>
-#include <string.h>
-
 #include <afs/stds.h>
+
+#ifdef IGNORE_SOME_GCC_WARNINGS
+# pragma GCC diagnostic warning "-Wstrict-prototypes"
+#endif
+
+#include <roken.h>
+
 #include <rx/rx.h>
 #include <rx/rxstat.h>
 #include "afs_kasAdmin.h"
@@ -363,7 +367,7 @@ kas_ServerOpen(const void *cellHandle, const char **serverList,
     afs_cell_handle_p c_handle = (afs_cell_handle_p) cellHandle;
     int server_count = 0, server_addr;
     struct afsconf_cell server_info;
-    kas_server_p k_handle = (kas_server_p) malloc(sizeof(kas_server_t));
+    kas_server_p k_handle = malloc(sizeof(kas_server_t));
 
     /*
      * Validate input parameters
@@ -918,10 +922,8 @@ kas_PrincipalGetBegin(const void *cellHandle, const void *serverHandle,
     afs_status_t tst = 0;
     afs_cell_handle_p c_handle = (afs_cell_handle_p) cellHandle;
     kas_server_p k_handle = (kas_server_p) serverHandle;
-    afs_admin_iterator_p iter =
-	(afs_admin_iterator_p) malloc(sizeof(afs_admin_iterator_t));
-    principal_get_p principal =
-	(principal_get_p) malloc(sizeof(principal_get_t));
+    afs_admin_iterator_p iter = malloc(sizeof(afs_admin_iterator_t));
+    principal_get_p principal = malloc(sizeof(principal_get_t));
 
     /*
      * Validate arguments
