@@ -9,20 +9,23 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
-#include <stdio.h>
-#include <ctype.h>
 #include <afs/stds.h>
+
+#include <roken.h>
+
+#include <ctype.h>
+
 #include <rx/rx.h>
 #include <rx/rxstat.h>
-#include "afs_ptsAdmin.h"
-#include "../adminutil/afs_AdminInternal.h"
+
 #include <afs/afs_AdminErrors.h>
 #include <afs/afs_utilAdmin.h>
 #include <afs/ptint.h>
 #include <afs/ptserver.h>
 
+#include "afs_ptsAdmin.h"
+#include "../adminutil/afs_AdminInternal.h"
+
 /*
  * IsValidCellHandle - validate the cell handle for making pts
  * requests.
@@ -339,7 +342,7 @@ TranslatePTSIds(const afs_cell_handle_p cellHandle, namelist * names,
     int rc = 0;
     afs_status_t tst = 0;
 
-    tst = ubik_PR_IDToName(cellHandle->pts, 0, ids, names);
+    tst = string_PR_IDToName(cellHandle->pts, 0, ids, names);
 
     if (tst) {
 	goto fail_TranslatePTSIds;
@@ -1150,7 +1153,7 @@ MemberListBegin(const void *cellHandle, const char *name, afs_status_t error1,
     afs_cell_handle_p c_handle = (afs_cell_handle_p) cellHandle;
     afs_int32 groupId = 0;
     afs_int32 exceeded = 0;
-    pts_group_member_list_iterator_p iter = (pts_group_member_list_iterator_p)
+    pts_group_member_list_iterator_p iter =
 	malloc(sizeof(pts_group_member_list_iterator_t));
     int iter_allocated = 0;
     int ids_allocated = 0;
@@ -2705,10 +2708,8 @@ pts_OwnedGroupListBegin(const void *cellHandle, const char *userName,
     int rc = 0;
     afs_status_t tst = 0;
     afs_cell_handle_p c_handle = (afs_cell_handle_p) cellHandle;
-    afs_admin_iterator_p iter =
-	(afs_admin_iterator_p) malloc(sizeof(afs_admin_iterator_t));
-    owned_group_list_p list =
-	(owned_group_list_p) malloc(sizeof(owned_group_list_t));
+    afs_admin_iterator_p iter = malloc(sizeof(afs_admin_iterator_t));
+    owned_group_list_p list = malloc(sizeof(owned_group_list_t));
 
     /*
      * Validate arguments
@@ -3042,9 +3043,8 @@ pts_UserListBegin(const void *cellHandle, void **iterationIdP,
     int rc = 0;
     afs_status_t tst = 0;
     afs_cell_handle_p c_handle = (afs_cell_handle_p) cellHandle;
-    afs_admin_iterator_p iter =
-	(afs_admin_iterator_p) malloc(sizeof(afs_admin_iterator_t));
-    pts_list_p list = (pts_list_p) malloc(sizeof(pts_list_t));
+    afs_admin_iterator_p iter = malloc(sizeof(afs_admin_iterator_t));
+    pts_list_p list = malloc(sizeof(pts_list_t));
 
     /*
      * Validate arguments
@@ -3226,9 +3226,8 @@ pts_GroupListBegin(const void *cellHandle, void **iterationIdP,
     int rc = 0;
     afs_status_t tst = 0;
     afs_cell_handle_p c_handle = (afs_cell_handle_p) cellHandle;
-    afs_admin_iterator_p iter =
-	(afs_admin_iterator_p) malloc(sizeof(afs_admin_iterator_t));
-    pts_list_p list = (pts_list_p) malloc(sizeof(pts_list_t));
+    afs_admin_iterator_p iter = malloc(sizeof(afs_admin_iterator_t));
+    pts_list_p list = malloc(sizeof(pts_list_t));
 
     /*
      * Validate arguments
