@@ -10,6 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
 #ifndef RXDEBUG
 char rxi_tracename[80] = "\0Tracing not compiled in";
@@ -21,18 +22,16 @@ main(int argc, char **argv)
 }
 #endif
 #else
-#include <string.h>
-#ifdef AFS_NT40_ENV
-#include <fcntl.h>
-#include <io.h>
-#else
-#include <sys/file.h>
-#include <unistd.h>
-#endif
+
 #include "rx.h"
+#include "rx_atomic.h"
 #include "rx_globals.h"
+#include "rx_internal.h"
 #include "rx_trace.h"
 
+#include "rx_conn.h"
+#include "rx_call.h"
+
 #ifdef RXTRACEON
 char rxi_tracename[80] = "/tmp/rxcalltrace";
 #else
@@ -61,8 +60,9 @@ rxi_flushtrace(void)
     rxi_tracepos = 0;
     if (rxi_logfd < 0)
 	return;
-    if (write(rxi_logfd, rxi_tracebuf, len) < 0)
-	; /* don't care */
+    if (write(rxi_logfd, rxi_tracebuf, len) < 0) {
+	/* don't care */
+    }
 }
 
 void
@@ -85,7 +85,7 @@ rxi_calltrace(unsigned int event, struct rx_call *call)
     rxtinfo.now = now.sec * 1000 + now.usec / 1000;
     rxtinfo.cid = call->conn->cid;
     rxtinfo.call = *(call->callNumber);
-    rxtinfo.qlen = rx_nWaiting;
+    rxtinfo.qlen = rx_atomic_read(&rx_nWaiting);
     rxtinfo.servicetime = 0;
     rxtinfo.waittime = 0;
 
@@ -133,7 +133,6 @@ rxi_calltrace(unsigned int event, struct rx_call *call)
 }
 
 #ifdef DUMPTRACE
-#include <errno.h>
 #ifdef AFS_NT40_ENV
 #include <afs/afsutil.h>
 #endif
