@@ -41,7 +41,8 @@ extern void outparam_conversion(afs_int32, char *, afs_int32);
 
 /* rmtsyss.c */
 extern void rmt_Quit(char *msg, ...)
-    AFS_ATTRIBUTE_FORMAT(__printf__, 1, 2);
+    AFS_ATTRIBUTE_FORMAT(__printf__, 1, 2)
+    AFS_NORETURN;
 
 extern void rmtsysd(void);
 
