@@ -111,6 +111,7 @@ extern int
 #define AFSDIR_CELLSERVDB_FILE  "CellServDB"
 #define AFSDIR_CELLALIAS_FILE   "CellAlias"
 #define AFSDIR_KEY_FILE         "KeyFile"
+#define AFSDIR_EXT_KEY_FILE	"KeyFileExt"
 #define AFSDIR_ULIST_FILE       "UserList"
 #define AFSDIR_NOAUTH_FILE      "NoAuth"
 #define AFSDIR_BUDBLOG_FILE     "BackupLog"
@@ -139,7 +140,7 @@ extern int
 #define AFSDIR_VOLSERLOG_FILE   "VolserLog"
 #define AFSDIR_AUDIT_FILE       "Audit"
 #define AFSDIR_KRB_EXCL_FILE    "krb.excl"
-#define AFSDIR_RXKAD_KEYTAB_FILE "rxkad.keytab"
+#define AFSDIR_RXKAD_KEYTAB_FILE	"rxkad.keytab"
 
 #define AFSDIR_ROOTVOL_FILE     "RootVolume"
 #define AFSDIR_HOSTDUMP_FILE    "hosts.dump"
@@ -157,6 +158,8 @@ extern int
 #else
 #define AFSDIR_CELLSERVDB_FILE_NTCLIENT  AFSDIR_CELLSERVDB_FILE
 #endif
+#define AFSDIR_CLIENT_CONFIG_FILE  "openafs-client.conf"
+#define AFSDIR_SERVER_CONFIG_FILE  "openafs-server.conf"
 
 #define AFSDIR_NETINFO_FILE     "NetInfo"
 #define AFSDIR_NETRESTRICT_FILE "NetRestrict"
@@ -275,85 +278,92 @@ typedef enum afsdir_id {
     AFSDIR_SERVER_SALSRV_FILEPATH_ID,
     AFSDIR_SERVER_SALSRVLOG_FILEPATH_ID,
     AFSDIR_SERVER_FSSTATE_FILEPATH_ID,
+    AFSDIR_CLIENT_CONFIG_FILE_FILEPATH_ID,
+    AFSDIR_SERVER_CONFIG_FILE_FILEPATH_ID,
     AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH_ID,
     AFSDIR_PATHSTRING_MAX
 } afsdir_id_t;
 
-/* getDirPath() returns a pointer to a string from an internal array of path strings
+/* afs_getDirPath() returns a pointer to a string from an internal array of path strings
  */
+const char *afs_getDirPath(afsdir_id_t string_id);
+/* For ABI compatibility with historical versions. */
 const char *getDirPath(afsdir_id_t string_id);
 
 /* Top level usr dir */
-#define AFSDIR_USR_DIRPATH getDirPath(AFSDIR_USR_DIRPATH_ID)
+#define AFSDIR_USR_DIRPATH afs_getDirPath(AFSDIR_USR_DIRPATH_ID)
 
 /* server subdir paths */
-#define AFSDIR_SERVER_AFS_DIRPATH getDirPath(AFSDIR_SERVER_AFS_DIRPATH_ID)
-#define AFSDIR_SERVER_ETC_DIRPATH getDirPath(AFSDIR_SERVER_ETC_DIRPATH_ID)
-#define AFSDIR_SERVER_BIN_DIRPATH getDirPath(AFSDIR_SERVER_BIN_DIRPATH_ID)
-#define AFSDIR_SERVER_CORES_DIRPATH getDirPath(AFSDIR_SERVER_CORES_DIRPATH_ID)
-#define AFSDIR_SERVER_DB_DIRPATH getDirPath(AFSDIR_SERVER_DB_DIRPATH_ID)
-#define AFSDIR_SERVER_LOGS_DIRPATH getDirPath(AFSDIR_SERVER_LOGS_DIRPATH_ID)
-#define AFSDIR_SERVER_LOCAL_DIRPATH getDirPath(AFSDIR_SERVER_LOCAL_DIRPATH_ID)
-#define AFSDIR_SERVER_BACKUP_DIRPATH getDirPath(AFSDIR_SERVER_BACKUP_DIRPATH_ID)
-#define AFSDIR_SERVER_LOCAL_DIRPATH getDirPath(AFSDIR_SERVER_LOCAL_DIRPATH_ID)
-#define AFSDIR_SERVER_MIGRATE_DIRPATH getDirPath(AFSDIR_SERVER_MIGRATE_DIRPATH_ID)
-#define AFSDIR_SERVER_MIGRATE_DIRPATH getDirPath(AFSDIR_SERVER_MIGRATE_DIRPATH_ID)
+#define AFSDIR_SERVER_AFS_DIRPATH afs_getDirPath(AFSDIR_SERVER_AFS_DIRPATH_ID)
+#define AFSDIR_SERVER_ETC_DIRPATH afs_getDirPath(AFSDIR_SERVER_ETC_DIRPATH_ID)
+#define AFSDIR_SERVER_BIN_DIRPATH afs_getDirPath(AFSDIR_SERVER_BIN_DIRPATH_ID)
+#define AFSDIR_SERVER_CORES_DIRPATH afs_getDirPath(AFSDIR_SERVER_CORES_DIRPATH_ID)
+#define AFSDIR_SERVER_DB_DIRPATH afs_getDirPath(AFSDIR_SERVER_DB_DIRPATH_ID)
+#define AFSDIR_SERVER_LOGS_DIRPATH afs_getDirPath(AFSDIR_SERVER_LOGS_DIRPATH_ID)
+#define AFSDIR_SERVER_LOCAL_DIRPATH afs_getDirPath(AFSDIR_SERVER_LOCAL_DIRPATH_ID)
+#define AFSDIR_SERVER_BACKUP_DIRPATH afs_getDirPath(AFSDIR_SERVER_BACKUP_DIRPATH_ID)
+#define AFSDIR_SERVER_LOCAL_DIRPATH afs_getDirPath(AFSDIR_SERVER_LOCAL_DIRPATH_ID)
+#define AFSDIR_SERVER_MIGRATE_DIRPATH afs_getDirPath(AFSDIR_SERVER_MIGRATE_DIRPATH_ID)
+#define AFSDIR_SERVER_MIGRATE_DIRPATH afs_getDirPath(AFSDIR_SERVER_MIGRATE_DIRPATH_ID)
 
 /* client subdir paths */
-#define AFSDIR_CLIENT_VICE_DIRPATH getDirPath(AFSDIR_CLIENT_VICE_DIRPATH_ID)
-#define AFSDIR_CLIENT_ETC_DIRPATH getDirPath(AFSDIR_CLIENT_ETC_DIRPATH_ID)
-#define AFSDIR_SERVER_BIN_FILE_DIRPATH getDirPath(AFSDIR_SERVER_BIN_FILE_DIRPATH_ID)
+#define AFSDIR_CLIENT_VICE_DIRPATH afs_getDirPath(AFSDIR_CLIENT_VICE_DIRPATH_ID)
+#define AFSDIR_CLIENT_ETC_DIRPATH afs_getDirPath(AFSDIR_CLIENT_ETC_DIRPATH_ID)
+#define AFSDIR_SERVER_BIN_FILE_DIRPATH afs_getDirPath(AFSDIR_SERVER_BIN_FILE_DIRPATH_ID)
 
 /* server file paths */
-#define AFSDIR_SERVER_THISCELL_FILEPATH getDirPath(AFSDIR_SERVER_THISCELL_FILEPATH_ID)
-#define AFSDIR_SERVER_CELLSERVDB_FILEPATH getDirPath(AFSDIR_SERVER_CELLSERVDB_FILEPATH_ID)
-#define AFSDIR_SERVER_NOAUTH_FILEPATH getDirPath(AFSDIR_SERVER_NOAUTH_FILEPATH_ID)
-#define AFSDIR_SERVER_KEY_FILEPATH getDirPath(AFSDIR_SERVER_KEY_FILEPATH_ID)
-#define AFSDIR_SERVER_ULIST_FILEPATH getDirPath(AFSDIR_SERVER_ULIST_FILEPATH_ID)
-#define AFSDIR_SERVER_BUDBLOG_FILEPATH getDirPath(AFSDIR_SERVER_BUDBLOG_FILEPATH_ID)
-#define AFSDIR_SERVER_TAPECONFIG_FILEPATH getDirPath(AFSDIR_SERVER_TAPECONFIG_FILEPATH_ID)
-#define AFSDIR_SERVER_KALOGDB_FILEPATH getDirPath(AFSDIR_SERVER_KALOGDB_FILEPATH_ID)
-#define AFSDIR_SERVER_KALOG_FILEPATH getDirPath(AFSDIR_SERVER_KALOG_FILEPATH_ID)
-#define AFSDIR_SERVER_KADB_FILEPATH getDirPath(AFSDIR_SERVER_KADB_FILEPATH_ID)
-#define AFSDIR_SERVER_NTPD_FILEPATH getDirPath(AFSDIR_SERVER_NTPD_FILEPATH_ID)
-#define AFSDIR_SERVER_PRDB_FILEPATH getDirPath(AFSDIR_SERVER_PRDB_FILEPATH_ID)
-#define AFSDIR_SERVER_PTLOG_FILEPATH getDirPath(AFSDIR_SERVER_PTLOG_FILEPATH_ID)
-#define AFSDIR_SERVER_KCONF_FILEPATH getDirPath(AFSDIR_SERVER_KCONF_FILEPATH_ID)
-#define AFSDIR_SERVER_VLDB_FILEPATH getDirPath(AFSDIR_SERVER_VLDB_FILEPATH_ID)
-#define AFSDIR_SERVER_VLOG_FILEPATH getDirPath(AFSDIR_SERVER_VLOG_FILEPATH_ID)
-#define AFSDIR_SERVER_CORELOG_FILEPATH getDirPath(AFSDIR_SERVER_CORELOG_FILEPATH_ID)
-#define AFSDIR_SERVER_SLVGLOG_FILEPATH getDirPath(AFSDIR_SERVER_SLVGLOG_FILEPATH_ID)
-#define AFSDIR_SERVER_SALSRVLOG_FILEPATH getDirPath(AFSDIR_SERVER_SALSRVLOG_FILEPATH_ID)
-#define AFSDIR_SERVER_SALVAGER_FILEPATH getDirPath(AFSDIR_SERVER_SALVAGER_FILEPATH_ID)
-#define AFSDIR_SERVER_SALSRV_FILEPATH getDirPath(AFSDIR_SERVER_SALSRV_FILEPATH_ID)
-#define AFSDIR_SERVER_BOZCONF_FILEPATH getDirPath(AFSDIR_SERVER_BOZCONF_FILEPATH_ID)
-#define AFSDIR_SERVER_BOZCONFNEW_FILEPATH getDirPath(AFSDIR_SERVER_BOZCONFNEW_FILEPATH_ID)
-#define AFSDIR_SERVER_BOZINIT_FILEPATH getDirPath(AFSDIR_SERVER_BOZINIT_FILEPATH_ID)
-#define AFSDIR_SERVER_BOZLOG_FILEPATH getDirPath(AFSDIR_SERVER_BOZLOG_FILEPATH_ID)
-#define AFSDIR_SERVER_BOZRXBIND_FILEPATH getDirPath(AFSDIR_SERVER_BOZRXBIND_FILEPATH_ID)
-#define AFSDIR_SERVER_BOSVR_FILEPATH getDirPath(AFSDIR_SERVER_BOSVR_FILEPATH_ID)
-#define AFSDIR_SERVER_SLVGLOCK_FILEPATH getDirPath(AFSDIR_SERVER_SLVGLOCK_FILEPATH_ID)
-#define AFSDIR_SERVER_VOLSERLOG_FILEPATH getDirPath(AFSDIR_SERVER_VOLSERLOG_FILEPATH_ID)
-#define AFSDIR_SERVER_ROOTVOL_FILEPATH getDirPath(AFSDIR_SERVER_ROOTVOL_FILEPATH_ID)
-#define AFSDIR_SERVER_HOSTDUMP_FILEPATH getDirPath(AFSDIR_SERVER_HOSTDUMP_FILEPATH_ID)
-#define AFSDIR_SERVER_CLNTDUMP_FILEPATH getDirPath(AFSDIR_SERVER_CLNTDUMP_FILEPATH_ID)
-#define AFSDIR_SERVER_CBKDUMP_FILEPATH getDirPath(AFSDIR_SERVER_CBKDUMP_FILEPATH_ID)
-#define AFSDIR_SERVER_OLDSYSID_FILEPATH getDirPath(AFSDIR_SERVER_OLDSYSID_FILEPATH_ID)
-#define AFSDIR_SERVER_SYSID_FILEPATH getDirPath(AFSDIR_SERVER_SYSID_FILEPATH_ID)
-#define AFSDIR_SERVER_FILELOG_FILEPATH getDirPath(AFSDIR_SERVER_FILELOG_FILEPATH_ID)
-#define AFSDIR_SERVER_AUDIT_FILEPATH getDirPath(AFSDIR_SERVER_AUDIT_FILEPATH_ID)
-#define AFSDIR_SERVER_NETINFO_FILEPATH getDirPath(AFSDIR_SERVER_NETINFO_FILEPATH_ID)
-#define AFSDIR_SERVER_NETRESTRICT_FILEPATH getDirPath(AFSDIR_SERVER_NETRESTRICT_FILEPATH_ID)
-#define AFSDIR_SERVER_WEIGHTING_CONSTANTS_FILEPATH getDirPath(AFSDIR_SERVER_WEIGHTING_CONSTANTS_FILEPATH_ID)
-#define AFSDIR_SERVER_THRESHOLD_CONSTANTS_FILEPATH getDirPath(AFSDIR_SERVER_THRESHOLD_CONSTANTS_FILEPATH_ID)
-#define AFSDIR_SERVER_MIGRATELOG_FILEPATH getDirPath(AFSDIR_SERVER_MIGRATELOG_FILEPATH_ID)
-#define AFSDIR_SERVER_KRB_EXCL_FILEPATH getDirPath(AFSDIR_SERVER_KRB_EXCL_FILEPATH_ID)
-#define AFSDIR_SERVER_FSSTATE_FILEPATH getDirPath(AFSDIR_SERVER_FSSTATE_FILEPATH_ID)
+#define AFSDIR_SERVER_THISCELL_FILEPATH afs_getDirPath(AFSDIR_SERVER_THISCELL_FILEPATH_ID)
+#define AFSDIR_SERVER_CELLSERVDB_FILEPATH afs_getDirPath(AFSDIR_SERVER_CELLSERVDB_FILEPATH_ID)
+#define AFSDIR_SERVER_NOAUTH_FILEPATH afs_getDirPath(AFSDIR_SERVER_NOAUTH_FILEPATH_ID)
+#define AFSDIR_SERVER_KEY_FILEPATH afs_getDirPath(AFSDIR_SERVER_KEY_FILEPATH_ID)
+#define AFSDIR_SERVER_ULIST_FILEPATH afs_getDirPath(AFSDIR_SERVER_ULIST_FILEPATH_ID)
+#define AFSDIR_SERVER_BUDBLOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_BUDBLOG_FILEPATH_ID)
+#define AFSDIR_SERVER_TAPECONFIG_FILEPATH afs_getDirPath(AFSDIR_SERVER_TAPECONFIG_FILEPATH_ID)
+#define AFSDIR_SERVER_KALOGDB_FILEPATH afs_getDirPath(AFSDIR_SERVER_KALOGDB_FILEPATH_ID)
+#define AFSDIR_SERVER_KALOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_KALOG_FILEPATH_ID)
+#define AFSDIR_SERVER_KADB_FILEPATH afs_getDirPath(AFSDIR_SERVER_KADB_FILEPATH_ID)
+#define AFSDIR_SERVER_NTPD_FILEPATH afs_getDirPath(AFSDIR_SERVER_NTPD_FILEPATH_ID)
+#define AFSDIR_SERVER_PRDB_FILEPATH afs_getDirPath(AFSDIR_SERVER_PRDB_FILEPATH_ID)
+#define AFSDIR_SERVER_PTLOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_PTLOG_FILEPATH_ID)
+#define AFSDIR_SERVER_KCONF_FILEPATH afs_getDirPath(AFSDIR_SERVER_KCONF_FILEPATH_ID)
+#define AFSDIR_SERVER_VLDB_FILEPATH afs_getDirPath(AFSDIR_SERVER_VLDB_FILEPATH_ID)
+#define AFSDIR_SERVER_VLOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_VLOG_FILEPATH_ID)
+#define AFSDIR_SERVER_CORELOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_CORELOG_FILEPATH_ID)
+#define AFSDIR_SERVER_SLVGLOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_SLVGLOG_FILEPATH_ID)
+#define AFSDIR_SERVER_SALSRVLOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_SALSRVLOG_FILEPATH_ID)
+#define AFSDIR_SERVER_SALVAGER_FILEPATH afs_getDirPath(AFSDIR_SERVER_SALVAGER_FILEPATH_ID)
+#define AFSDIR_SERVER_SALSRV_FILEPATH afs_getDirPath(AFSDIR_SERVER_SALSRV_FILEPATH_ID)
+#define AFSDIR_SERVER_BOZCONF_FILEPATH afs_getDirPath(AFSDIR_SERVER_BOZCONF_FILEPATH_ID)
+#define AFSDIR_SERVER_BOZCONFNEW_FILEPATH afs_getDirPath(AFSDIR_SERVER_BOZCONFNEW_FILEPATH_ID)
+#define AFSDIR_SERVER_BOZINIT_FILEPATH afs_getDirPath(AFSDIR_SERVER_BOZINIT_FILEPATH_ID)
+#define AFSDIR_SERVER_BOZLOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_BOZLOG_FILEPATH_ID)
+#define AFSDIR_SERVER_BOZRXBIND_FILEPATH afs_getDirPath(AFSDIR_SERVER_BOZRXBIND_FILEPATH_ID)
+#define AFSDIR_SERVER_BOSVR_FILEPATH afs_getDirPath(AFSDIR_SERVER_BOSVR_FILEPATH_ID)
+#define AFSDIR_SERVER_SLVGLOCK_FILEPATH afs_getDirPath(AFSDIR_SERVER_SLVGLOCK_FILEPATH_ID)
+#define AFSDIR_SERVER_VOLSERLOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_VOLSERLOG_FILEPATH_ID)
+#define AFSDIR_SERVER_ROOTVOL_FILEPATH afs_getDirPath(AFSDIR_SERVER_ROOTVOL_FILEPATH_ID)
+#define AFSDIR_SERVER_HOSTDUMP_FILEPATH afs_getDirPath(AFSDIR_SERVER_HOSTDUMP_FILEPATH_ID)
+#define AFSDIR_SERVER_CLNTDUMP_FILEPATH afs_getDirPath(AFSDIR_SERVER_CLNTDUMP_FILEPATH_ID)
+#define AFSDIR_SERVER_CBKDUMP_FILEPATH afs_getDirPath(AFSDIR_SERVER_CBKDUMP_FILEPATH_ID)
+#define AFSDIR_SERVER_OLDSYSID_FILEPATH afs_getDirPath(AFSDIR_SERVER_OLDSYSID_FILEPATH_ID)
+#define AFSDIR_SERVER_SYSID_FILEPATH afs_getDirPath(AFSDIR_SERVER_SYSID_FILEPATH_ID)
+#define AFSDIR_SERVER_FILELOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_FILELOG_FILEPATH_ID)
+#define AFSDIR_SERVER_AUDIT_FILEPATH afs_getDirPath(AFSDIR_SERVER_AUDIT_FILEPATH_ID)
+#define AFSDIR_SERVER_NETINFO_FILEPATH afs_getDirPath(AFSDIR_SERVER_NETINFO_FILEPATH_ID)
+#define AFSDIR_SERVER_NETRESTRICT_FILEPATH afs_getDirPath(AFSDIR_SERVER_NETRESTRICT_FILEPATH_ID)
+#define AFSDIR_SERVER_WEIGHTING_CONSTANTS_FILEPATH afs_getDirPath(AFSDIR_SERVER_WEIGHTING_CONSTANTS_FILEPATH_ID)
+#define AFSDIR_SERVER_THRESHOLD_CONSTANTS_FILEPATH afs_getDirPath(AFSDIR_SERVER_THRESHOLD_CONSTANTS_FILEPATH_ID)
+#define AFSDIR_SERVER_MIGRATELOG_FILEPATH afs_getDirPath(AFSDIR_SERVER_MIGRATELOG_FILEPATH_ID)
+#define AFSDIR_SERVER_KRB_EXCL_FILEPATH afs_getDirPath(AFSDIR_SERVER_KRB_EXCL_FILEPATH_ID)
+#define AFSDIR_SERVER_FSSTATE_FILEPATH afs_getDirPath(AFSDIR_SERVER_FSSTATE_FILEPATH_ID)
+#define AFSDIR_SERVER_CONFIG_FILE_FILEPATH afs_getDirPath(AFSDIR_SERVER_CONFIG_FILE_FILEPATH_ID)
+#define AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH afs_getDirPath(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH_ID)
 
 /* client file paths */
-#define AFSDIR_CLIENT_THISCELL_FILEPATH getDirPath(AFSDIR_CLIENT_THISCELL_FILEPATH_ID)
-#define AFSDIR_CLIENT_CELLSERVDB_FILEPATH getDirPath(AFSDIR_CLIENT_CELLSERVDB_FILEPATH_ID)
-#define AFSDIR_CLIENT_NETINFO_FILEPATH getDirPath(AFSDIR_CLIENT_NETINFO_FILEPATH_ID)
-#define AFSDIR_CLIENT_NETRESTRICT_FILEPATH getDirPath(AFSDIR_CLIENT_NETRESTRICT_FILEPATH_ID)
+#define AFSDIR_CLIENT_CONFIG_FILE_FILEPATH afs_getDirPath(AFSDIR_CLIENT_CONFIG_FILE_FILEPATH_ID)
+#define AFSDIR_CLIENT_THISCELL_FILEPATH afs_getDirPath(AFSDIR_CLIENT_THISCELL_FILEPATH_ID)
+#define AFSDIR_CLIENT_CELLSERVDB_FILEPATH afs_getDirPath(AFSDIR_CLIENT_CELLSERVDB_FILEPATH_ID)
+#define AFSDIR_CLIENT_NETINFO_FILEPATH afs_getDirPath(AFSDIR_CLIENT_NETINFO_FILEPATH_ID)
+#define AFSDIR_CLIENT_NETRESTRICT_FILEPATH afs_getDirPath(AFSDIR_CLIENT_NETRESTRICT_FILEPATH_ID)
 
 #endif /* _DIRPATH_H */
