@@ -9,16 +9,12 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#include <sys/types.h>
-#include <errno.h>
-#include <stdio.h>
-#include <sys/file.h>
-#include <string.h>
 
-#include <netinet/in.h>
+#include <roken.h>
+
+#include <sys/file.h>
+
 #include <afs/venus.h>
 #include <afs/cmd.h>
 #include <afs/afsutil.h>
@@ -265,7 +261,7 @@ handleit(struct cmd_syndesc *as, void *arock)
     }
     close(new);
 
-    renamefile(tempname, pathname);
+    rk_rename(tempname, pathname);
     sleep(5);
     exit(0);
 }
@@ -411,7 +407,7 @@ read_mhentries(afs_uint32 mh_addr, int oldfd)
 	perror("seek MH block");
 	exit(1);
     }
-    base[0] = (struct extentaddr *)malloc(VL_ADDREXTBLK_SIZE);
+    base[0] = malloc(VL_ADDREXTBLK_SIZE);
     if (!base[0]) {
 	perror("malloc1");
 	exit(1);
@@ -425,7 +421,7 @@ read_mhentries(afs_uint32 mh_addr, int oldfd)
     }
 
     /* Verify that this block is the right one */
-    if (ntohl(base[0]->ex_flags) != VLCONTBLOCK) {	/* check if flag is correct */
+    if (ntohl(base[0]->ex_hdrflags) != VLCONTBLOCK) {	/* check if flag is correct */
 	free(base[0]);
 	base[0] = 0;
 	return;
@@ -456,7 +452,7 @@ read_mhentries(afs_uint32 mh_addr, int oldfd)
 	    perror("seek MH block");
 	    exit(1);
 	}
-	base[j] = (struct extentaddr *)malloc(VL_ADDREXTBLK_SIZE);
+	base[j] = malloc(VL_ADDREXTBLK_SIZE);
 	if (!base[j]) {
 	    perror("malloc1");
 	    exit(1);
@@ -468,7 +464,7 @@ read_mhentries(afs_uint32 mh_addr, int oldfd)
 	}
 
 	/* Verify that this block knows its an extent block */
-	if (ntohl(base[j]->ex_flags) != VLCONTBLOCK) {
+	if (ntohl(base[j]->ex_hdrflags) != VLCONTBLOCK) {
 	    free(base[j]);
 	    base[j] = 0;
 	    continue;
@@ -1004,7 +1000,7 @@ main(int argc, char **argv)
     struct cmd_syndesc *ts;
     afs_int32 code;
 
-    ts = cmd_CreateSyntax("initcmd", handleit, NULL, "optional");
+    ts = cmd_CreateSyntax("initcmd", handleit, NULL, 0, "optional");
     cmd_AddParm(ts, "-to", CMD_SINGLE, CMD_OPTIONAL, "goal version");
     cmd_AddParm(ts, "-from", CMD_SINGLE, CMD_OPTIONAL, "current version");
     cmd_AddParm(ts, "-path", CMD_SINGLE, CMD_OPTIONAL, "pathname");
