@@ -42,28 +42,19 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
 #include <afs/afsint.h>
 #include <afs/nfs.h>
+#include <rx/rx_queue.h>
 #include <lock.h>
 #include <afs/ihandle.h>
 #include <afs/vnode.h>
 #include <afs/volume.h>
-#include "volint.h"
-#include "dump.h"
 #include <afs/cmd.h>
 
-#include <sys/param.h>
-#include <sys/types.h>
-#include <sys/uio.h>
-#include <stdio.h>
-#include <errno.h>
-#include <netinet/in.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <dirent.h>
-#include <string.h>
-
+#include "volint.h"
+#include "dump.h"
 
 char rootdir[MAXPATHLEN];
 char mntroot[MAXPATHLEN];
@@ -86,8 +77,8 @@ readvalue(int size)
     ptr = (char *)&value;
 
     s = sizeof(value) - size;
-    if (size < 0) {
-	fprintf(stderr, "Too much data in afs_int32\n");
+    if (s < 0) {
+	fprintf(stderr, "Too much data for afs_int32\n");
 	return 0;
     }
 
@@ -389,15 +380,17 @@ struct vNode {
 
 #define MAXNAMELEN 256
 
-afs_int32
-ReadVNode(afs_int32 count)
+static int
+ReadVNode(afs_int32 count, afs_int32 *return_tag)
 {
     struct vNode vn;
     int code, i, done;
     char tag;
-    char dirname[MAXNAMELEN], linkname[MAXNAMELEN], lname[MAXNAMELEN];
-    char parentdir[MAXNAMELEN], vflink[MAXNAMELEN];
-    char filename[MAXNAMELEN], fname[MAXNAMELEN];
+    char *dirname = NULL, *lname = NULL, *slinkname = NULL;
+    char linkname[MAXNAMELEN+1];
+    char *parentdir = NULL, *vflink = NULL;
+    char *filename = NULL;
+    char fname[MAXNAMELEN+1];
     int len;
     afs_int32 vnode;
     afs_int32 mode = 0;
@@ -476,35 +469,57 @@ ReadVNode(afs_int32 count)
 	     * The parent dir and symbolic link to it must exist.
 	     */
 	    vnode = ((vn.type == 2) ? vn.vnode : vn.parent);
-	    if (vnode == 1)
-		strncpy(parentdir, rootdir, sizeof parentdir);
-	    else {
-		afs_snprintf(parentdir, sizeof parentdir, "%s" OS_DIRSEP "%s%d", rootdir,
-			     ADIR, vnode);
+	    if (vnode == 1) {
+		free(parentdir);
+		parentdir = strdup(rootdir);
+		if (!parentdir) {
+		    code = ENOMEM;
+		    goto common_exit;
+		}
+	    } else {
+		free(parentdir);
+		if (asprintf(&parentdir,
+			 "%s" OS_DIRSEP "%s%d", rootdir, ADIR, vnode) < 0) {
+		    parentdir = NULL;
+		    code = ENOMEM;
+		    goto common_exit;
+		}
 
 		len = readlink(parentdir, linkname, MAXNAMELEN);
 		if (len < 0) {
 		    /* parentdir does not exist. So create an orphan dir.
 		     * and then link the parentdir to the orphaned dir.
 		     */
-		    afs_snprintf(linkname, sizeof linkname, "%s" OS_DIRSEP "%s%d",
-				 rootdir, ODIR, vnode);
-		    code = mkdir(linkname, 0777);
+		    if (asprintf(&lname, "%s" OS_DIRSEP "%s%d",
+			     rootdir, ODIR, vnode) < 0) {
+			lname = NULL;
+			code = ENOMEM;
+			goto common_exit;
+		    }
+
+		    code = mkdir(lname, 0777);
 		    if ((code < 0) && (errno != EEXIST)) {
 			fprintf(stderr,
 				"Error creating directory %s  code=%d;%d\n",
-				linkname, code, errno);
+				lname, code, errno);
+		    }
+		    free(lname);
+		    /* Link the parentdir to it - now parentdir exists */
+		    if (asprintf(&lname, "%s%d/", ODIR,
+			     vnode) < 0) {
+			lname = NULL;
+			code = ENOMEM;
+			goto common_exit;
 		    }
 
-		    /* Link the parentdir to it - now parentdir exists */
-		    afs_snprintf(linkname, sizeof linkname, "%s%d/", ODIR,
-				 vnode);
-		    code = symlink(linkname, parentdir);
+		    code = symlink(lname, parentdir);
 		    if (code) {
 			fprintf(stderr,
 				"Error creating symlink %s -> %s  code=%d;%d\n",
-				parentdir, linkname, code, errno);
+				parentdir, lname, code, errno);
 		    }
+		    free(lname);
+		    lname = NULL;
 		}
 	    }
 
@@ -552,7 +567,11 @@ ReadVNode(afs_int32 count)
 
 
 		buffer = NULL;
-		buffer = (char *)malloc(vn.dataSize);
+		buffer = calloc(1, vn.dataSize);
+		if (!buffer) {
+		    code = ENOMEM;
+		    goto common_exit;
+		}
 
 		readdata(buffer, vn.dataSize);
 		page0 = (struct Page0 *)buffer;
@@ -580,10 +599,22 @@ ReadVNode(afs_int32 count)
 				/* dirname is the directory to create.
 				 * vflink is what will link to it.
 				 */
-				afs_snprintf(dirname, sizeof dirname, "%s" OS_DIRSEP "%s",
-					     parentdir, this_name);
-			    afs_snprintf(vflink, sizeof vflink, "%s" OS_DIRSEP "%s%d",
-					 rootdir, ADIR, this_vn);
+			    if (asprintf(&dirname,
+				     "%s" OS_DIRSEP "%s",
+				     parentdir, this_name) < 0) {
+				free(buffer);
+				code = ENOMEM;
+				goto common_exit;
+			    }
+
+			    if (asprintf(&vflink,
+				     "%s" OS_DIRSEP "%s%d",
+				     rootdir, ADIR, this_vn) < 0) {
+				free(buffer);
+				free(dirname);
+				code = ENOMEM;
+				goto common_exit;
+			    }
 
 			    /* The link and directory may already exist */
 			    len = readlink(vflink, linkname, MAXNAMELEN);
@@ -602,24 +633,48 @@ ReadVNode(afs_int32 count)
 				 * It was created originally as orphaned.
 				 */
 				linkname[len - 1] = '\0';	/* remove '/' at end */
-				afs_snprintf(lname, sizeof lname, "%s" OS_DIRSEP "%s",
-					     rootdir, linkname);
+				if (asprintf(&lname,
+				         "%s" OS_DIRSEP "%s",
+					 rootdir, linkname) < 0) {
+				    free(buffer);
+				    free(dirname);
+				    free(vflink);
+				    lname = NULL;
+				    code = ENOMEM;
+				    goto common_exit;
+				}
+
 				code = rename(lname, dirname);
 				if (code) {
 				    fprintf(stderr,
 					    "Error renaming %s to %s  code=%d;%d\n",
 					    lname, dirname, code, errno);
 				}
+				free(lname);
+				lname = NULL;
 			    }
+			    free(dirname);
+			    dirname = NULL;
 
 			    /* Now create/update the link to the new/moved directory */
-			    if (vn.vnode == 1)
-				afs_snprintf(dirname, sizeof dirname, "%s/",
-					     this_name);
-			    else
-				afs_snprintf(dirname, sizeof dirname,
-					     "%s%d/%s/", ADIR, vn.vnode,
-					     this_name);
+			    if (vn.vnode == 1) {
+				if (asprintf(&dirname, "%s/",
+					 this_name) < 0) {
+				    free(buffer);
+				    free(vflink);
+				    code = ENOMEM;
+				    goto common_exit;
+				}
+			    } else {
+				if (asprintf(&dirname, "%s%d/%s/",
+					 ADIR, vn.vnode, this_name) < 0) {
+				    free(buffer);
+				    free(vflink);
+				    code = ENOMEM;
+				    goto common_exit;
+				}
+			    }
+
 			    unlink(vflink);
 			    code = symlink(dirname, vflink);
 			    if (code) {
@@ -627,17 +682,25 @@ ReadVNode(afs_int32 count)
 					"Error creating symlink %s -> %s  code=%d;%d\n",
 					vflink, dirname, code, errno);
 			    }
+			    free(dirname);
+			    free(vflink);
+			    dirname = NULL;
+			    vflink = NULL;
 			}
 			/*ADIRENTRY*/
 			    /* For a file entry, we remember the name of the file
 			     * by creating a link within the directory. Restoring
 			     * the file will later remove the link.
 			     */
-			    else {
-			     /*AFILEENTRY*/ afs_snprintf(vflink,
-							 sizeof vflink,
-							 "%s" OS_DIRSEP "%s%d", parentdir,
-							 AFILE, this_vn);
+			else {
+			     /*AFILEENTRY*/
+			    if (asprintf(&vflink,
+				     "%s" OS_DIRSEP "%s%d", parentdir,
+				     AFILE, this_vn) < 0) {
+				free(buffer);
+				code = ENOMEM;
+				goto common_exit;
+			    }
 
 			    code = symlink(this_name, vflink);
 			    if ((code < 0) && (errno != EEXIST)) {
@@ -646,10 +709,13 @@ ReadVNode(afs_int32 count)
 					vflink, page0->entry[j].name, code,
 					errno);
 			    }
+			    free(vflink);
+			    vflink = NULL;
 			}
 		     /*AFILEENTRY*/}
 		}
 		free(buffer);
+		buffer = NULL;
 	    }
 	    /*ITSADIR*/
 	    else if (vn.type == 1) {
@@ -660,17 +726,25 @@ ReadVNode(afs_int32 count)
 		int fid;
 		int lfile;
 		afs_sfsize_t size, s;
+		ssize_t count;
 
 		/* Check if its vnode-file-link exists. If not,
 		 * then the file will be an orphaned file.
 		 */
 		lfile = 1;
-		afs_snprintf(filename, sizeof filename, "%s" OS_DIRSEP "%s%d", parentdir,
-			     AFILE, vn.vnode);
+		if (asprintf(&filename, "%s" OS_DIRSEP "%s%d",
+			 parentdir, AFILE, vn.vnode) < 0) {
+		    code = ENOMEM;
+		    goto common_exit;
+		}
 		len = readlink(filename, fname, MAXNAMELEN);
 		if (len < 0) {
-		    afs_snprintf(filename, sizeof filename, "%s" OS_DIRSEP "%s%d",
-				 rootdir, OFILE, vn.vnode);
+		    free(filename);
+		    if (asprintf(&filename, "%s" OS_DIRSEP "%s%d",
+			     rootdir, OFILE, vn.vnode) < 0) {
+			code = ENOMEM;
+			goto common_exit;
+		    }
 		    lfile = 0;	/* no longer a linked file; a direct path */
 		}
 
@@ -683,29 +757,39 @@ ReadVNode(afs_int32 count)
 
 		/* Write the file out */
 		fid = open(filename, (O_CREAT | O_WRONLY | O_TRUNC), mode);
+		if (fid < 0) {
+		    fprintf(stderr, "Open %s: Errno = %d\n", filename, errno);
+		    goto open_fail;
+		}
 		size = vn.dataSize;
 		while (size > 0) {
 		    s = (afs_int32) ((size > BUFSIZE) ? BUFSIZE : size);
 		    code = fread(buf, 1, s, dumpfile);
-		    if (code > 0) {
-			(void)write(fid, buf, code);
-			size -= code;
-		    }
 		    if (code != s) {
 			if (code < 0)
 			    fprintf(stderr, "Code = %d; Errno = %d\n", code,
 				    errno);
 			else {
-			    char tmp[100];
-			    (void)afs_snprintf(tmp, sizeof tmp,
-					       "Read %llu bytes out of %llu",
-					       (afs_uintmax_t) (vn.dataSize -
-								size),
-					       (afs_uintmax_t) vn.dataSize);
-			    fprintf(stderr, "%s\n", tmp);
+			    fprintf(stderr, "Read %llu bytes out of %llu\n",
+				     (afs_uintmax_t) (vn.dataSize - size),
+				     (afs_uintmax_t) vn.dataSize);
 			}
 			break;
 		    }
+		    if (code > 0) {
+			count = write(fid, buf, code);
+			if (count < 0) {
+			    fprintf(stderr, "Count = %ld, Errno = %d\n",
+				    (long)count, errno);
+			    break;
+			} else if (count != code) {
+			    fprintf(stderr, "Wrote %llu bytes out of %llu\n",
+				    (afs_uintmax_t) count,
+				    (afs_uintmax_t) code);
+			    break;
+			}
+			size -= code;
+		    }
 		}
 		close(fid);
 		if (size != 0) {
@@ -713,10 +797,13 @@ ReadVNode(afs_int32 count)
 			    filename, fname);
 		}
 
+open_fail:
 		/* Remove the link to the file */
 		if (lfile) {
 		    unlink(filename);
 		}
+		free(filename);
+		filename = NULL;
 	    }
 	    /*ITSAFILE*/
 	    else if (vn.type == 3) {
@@ -731,16 +818,28 @@ ReadVNode(afs_int32 count)
 		 * of the symbolic link. If it doesn't exist,
 		 * then the link will be an orphaned link.
 		 */
-		afs_snprintf(linkname, sizeof linkname, "%s" OS_DIRSEP "%s%d", parentdir,
-			     AFILE, vn.vnode);
-		len = readlink(linkname, fname, MAXNAMELEN - 1);
+		if (asprintf(&slinkname, "%s" OS_DIRSEP "%s%d",
+			 parentdir, AFILE, vn.vnode) < 0) {
+		    code = ENOMEM;
+		    goto common_exit;
+		}
+
+		len = readlink(slinkname, fname, MAXNAMELEN);
 		if (len < 0) {
-		    afs_snprintf(filename, sizeof filename, "%s" OS_DIRSEP "%s%d",
-				 rootdir, OFILE, vn.vnode);
+		    if (asprintf(&filename, "%s" OS_DIRSEP "%s%d",
+			     rootdir, OFILE, vn.vnode) < 0) {
+			free(slinkname);
+			code = ENOMEM;
+			goto common_exit;
+		    }
 		} else {
 		    fname[len] = '\0';
-		    afs_snprintf(filename, sizeof filename, "%s" OS_DIRSEP "%s",
-				 parentdir, fname);
+		    if (asprintf(&filename, "%s" OS_DIRSEP "%s",
+			     parentdir, fname) < 0) {
+			free(slinkname);
+			code = ENOMEM;
+			goto common_exit;
+		    }
 		}
 
 		/* Read the link in, delete it, and then create it */
@@ -752,9 +851,27 @@ ReadVNode(afs_int32 count)
 		    && (buf[s - 1] == '.')) {
 		    /* This is a symbolic link */
 		    buf[s - 1] = 0;	/* Remove prefix '.' */
-		    strcpy(lname, &buf[1]);	/* Remove postfix '#' or '%' */
-		    strcpy(buf, mntroot);
-		    strcat(buf, lname);
+		    lname = strdup(&buf[1]);	/* Remove postfix '#' or '%' */
+		    if (!lname) {
+			free(filename);
+			free(slinkname);
+			code = ENOMEM;
+			goto common_exit;
+		    }
+		    if (strlcpy(buf, mntroot, BUFSIZE) >= BUFSIZE) {
+			free(filename);
+			free(slinkname);
+			code = EMSGSIZE;
+			goto common_exit;
+		    }
+		    if (strlcat(buf, lname, BUFSIZE) >= BUFSIZE) {
+			free(filename);
+			free(slinkname);
+			code = EMSGSIZE;
+			goto common_exit;
+		    }
+		    free(lname);
+		    lname = NULL;
 		}
 
 		unlink(filename);
@@ -766,7 +883,12 @@ ReadVNode(afs_int32 count)
 		}
 
 		/* Remove the symbolic link */
-		unlink(linkname);
+		unlink(slinkname);
+		free(slinkname);
+		free(filename);
+		slinkname = NULL;
+		filename = NULL;
+
 	    }
 	    /*ITSASYMLINK*/
 	    else {
@@ -782,7 +904,13 @@ ReadVNode(afs_int32 count)
     if (vn.type == 0)
 	inc_dump = 1;
 
-    return ((afs_int32) tag);
+
+    *return_tag  = (afs_int32)tag;
+    code = 0;
+  common_exit:
+    free(parentdir);
+    free(lname);
+    return code;
 }
 
 static int
@@ -792,13 +920,16 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
     afs_int32 type, count, vcount;
     DIR *dirP, *dirQ;
     struct dirent *dirE, *dirF;
-    char name[MAXNAMELEN];
-    char thisdir[MAXPATHLEN], *t;
+    char *name = NULL;
+    char *thisdir, *t;
     struct DumpHeader dh;	/* Defined in dump.h */
 #if 0/*ndef HAVE_GETCWD*/	/* XXX enable when autoconf happens */
     extern char *getwd();
 #define getcwd(x,y) getwd(x)
 #endif
+    thisdir = calloc(1, MAXPATHLEN+4);
+    if (!thisdir)
+	goto mem_error_exit;
 
     if (as->parms[0].items) {	/* -file <dumpfile> */
 	dumpfile = fopen(as->parms[0].items->data, "r");
@@ -822,14 +953,17 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 
     /* Get the root directory we restore to */
     if (as->parms[1].items) {	/* -dir <rootdir> */
-	strcpy(rootdir, as->parms[1].items->data);
+	if (strlcpy(rootdir, as->parms[1].items->data, sizeof(rootdir)) >= sizeof(rootdir))
+	    goto str_error_exit;
     } else {
 	strcpy(rootdir, ".");
     }
-    strcat(rootdir, "/");
+    if (strlcat(rootdir, "/", sizeof(rootdir)) >= sizeof(rootdir))
+	goto str_error_exit;
 
     /* Append the RW volume name to the root directory */
-    strcat(rootdir, dh.volumeName);
+    if (strlcat(rootdir, dh.volumeName, sizeof(rootdir)) >= sizeof(rootdir))
+	goto str_error_exit;
     len = strlen(rootdir);
     if (strcmp(".backup", rootdir + len - 7) == 0) {
 	rootdir[len - 7] = 0;
@@ -839,7 +973,8 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 
     /* Append the extension we asked for */
     if (as->parms[2].items) {
-	strcat(rootdir, as->parms[2].items->data);	/* -extension <ext> */
+	if (strlcat(rootdir, as->parms[2].items->data, sizeof(rootdir)) >= sizeof(rootdir))    /* -extension <ext> */
+	    goto str_error_exit;
     }
 
     /* The mountpoint root is either specifid in -mountpoint
@@ -871,7 +1006,8 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 	    code = -1;
 	    goto cleanup;
 	}
-	strcat(mntroot, "/");	/* append '/' to end of it */
+	if (strlcat(mntroot, "/", sizeof(mntroot)) >= sizeof(mntroot))	/* append '/' to end of it */
+	    goto str_error_exit;
 	code = chdir(thisdir);	/* return to original working dir */
 	if (code) {
 	    fprintf(stderr, "Cannot find working directory: Error = %d\n",
@@ -888,7 +1024,8 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 	    goto cleanup;
 	}
     }
-    strcat(mntroot, "/");	/* append '/' to end of it */
+    if (strlcat(mntroot, "/", sizeof(mntroot)) >= sizeof(mntroot))	/* append '/' to end of it */
+	goto str_error_exit;
 
     /* Set the umask for the restore */
     if (as->parms[4].items) {	/* -umask */
@@ -908,8 +1045,13 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 
     for (count = 1; type == 2; count++) {
 	type = ReadVolumeHeader(count);
-	for (vcount = 1; type == 3; vcount++)
-	    type = ReadVNode(vcount);
+	for (vcount = 1; type == 3; vcount++) {
+	    code = ReadVNode(vcount, &type);
+	    if (code == ENOMEM)
+		goto mem_error_exit;
+	    if (code == EMSGSIZE)
+		goto str_error_exit;
+	}
     }
 
     if (type != 4) {
@@ -927,37 +1069,70 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 	dirP = opendir(rootdir);
 	while (dirP && (dirE = readdir(dirP))) {
 	    if (strncmp(dirE->d_name, ADIR, strlen(ADIR)) == 0) {
-		afs_snprintf(name, sizeof name, "%s" OS_DIRSEP "%s", rootdir,
-			     dirE->d_name);
+		if (asprintf(&name, "%s" OS_DIRSEP "%s", rootdir,
+			 dirE->d_name) < 0)
+		    goto mem_error_exit;
+
 		dirQ = opendir(name);
+		free(name);
+		name = NULL;
 		while (dirQ && (dirF = readdir(dirQ))) {
 		    if (strncmp(dirF->d_name, AFILE, strlen(AFILE)) == 0) {
-			afs_snprintf(name, sizeof name, "%s" OS_DIRSEP "%s/%s", rootdir,
-				     dirE->d_name, dirF->d_name);
+			if (asprintf(&name, "%s" OS_DIRSEP "%s/%s",
+				 rootdir, dirE->d_name, dirF->d_name) < 0)
+			    goto mem_error_exit;
+
 			unlink(name);
+			free(name);
+			name = NULL;
 		    }
 		}
-		closedir(dirQ);
+		if (dirQ)
+		    closedir(dirQ);
 	    } else if (strncmp(dirE->d_name, AFILE, strlen(AFILE)) == 0) {
-		afs_snprintf(name, sizeof name, "%s" OS_DIRSEP "%s", rootdir,
-			     dirE->d_name);
+		if (asprintf(&name, "%s" OS_DIRSEP "%s", rootdir,
+			 dirE->d_name) < 0)
+		    goto mem_error_exit;
+
 		unlink(name);
+		free(name);
+		name = NULL;
 	    }
 	}
-	closedir(dirP);
+	if (dirP)
+	    closedir(dirP);
     }
 
     /* Now go through and remove all the directory links */
     dirP = opendir(rootdir);
     while (dirP && (dirE = readdir(dirP))) {
 	if (strncmp(dirE->d_name, ADIR, strlen(ADIR)) == 0) {
-	    afs_snprintf(name, sizeof name, "%s" OS_DIRSEP "%s", rootdir, dirE->d_name);
+	    if (asprintf(&name, "%s" OS_DIRSEP "%s", rootdir,
+		     dirE->d_name) < 0)
+		goto mem_error_exit;
+
 	    unlink(name);
+	    free(name);
+	    name = NULL;
 	}
     }
-    closedir(dirP);
+    if (dirP)
+	closedir(dirP);
 
+    free(thisdir);
     return (code);
+
+  mem_error_exit:
+    /* Error allocating memory -- quick exit */
+    fprintf(stderr, "Memory allocation error!\n");
+    free(thisdir);
+    return -1;
+
+  str_error_exit:
+    /* Str length error */
+    fprintf(stderr, "String exceeded buffer size\n");
+    free(thisdir);
+    return -1;
 }
 
 int
@@ -967,7 +1142,7 @@ main(int argc, char **argv)
 
     setlinebuf(stdout);
 
-    ts = cmd_CreateSyntax(NULL, WorkerBee, NULL, "vldb check");
+    ts = cmd_CreateSyntax(NULL, WorkerBee, NULL, 0, "vldb check");
     cmd_AddParm(ts, "-file", CMD_SINGLE, CMD_OPTIONAL, "dump file");
     cmd_AddParm(ts, "-dir", CMD_SINGLE, CMD_OPTIONAL, "restore dir");
     cmd_AddParm(ts, "-extension", CMD_SINGLE, CMD_OPTIONAL, "name extension");
